"""
股票改概念板块处理
"""
from WindPy import w
import pandas as pd
import hbshare as hbs
import plotly.figure_factory as ff
import plotly
from plotly.offline import plot as plot_ly
import plotly.graph_objs as go

plotly.offline.init_notebook_mode(connected=True)

w.start()


# 以20220128为例
date = '20220819'
sql_script = "SELECT SYMBOL, TDATE, VATURNOVER, TCAP FROM finchina.CHDQUOTE WHERE TDATE = {}".format(date)
data = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
data = data[data['SYMBOL'].str[0].isin(['0', '3', '6'])]
data = data[data['VATURNOVER'] > 1e-6]
data['sec_code'] = data['SYMBOL'].apply(lambda x: x + '.SH' if x[0] == '6' else x + '.SZ')
ticker_list = data['sec_code'].tolist()
res = w.wss(','.join(ticker_list), 'concept', "tradeDate={}".format(date))
if res.ErrorCode != 0:
    df = pd.DataFrame()
    print("fetch concept data error: trade_date = {}".format(date))
else:
    df = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Fields)
    df.index.name = 'sec_code'
# 处理
concept_list = []
for sec in df.index.tolist():
    sub_list = df.loc[sec, 'CONCEPT'].split(';')
    concept_list = list(set(concept_list).union(set(sub_list)))

concept_df = pd.DataFrame(index=df.index, columns=concept_list)
for sec in concept_df.index.tolist():
    sub_list = df.loc[sec, 'CONCEPT'].split(';')
    concept_df.loc[sec, sub_list] = 1
# 每个概念的只数和市值统计
concept_df = pd.merge(concept_df, data[['sec_code', 'TCAP']], left_index=True, right_on='sec_code')
count_df = pd.DataFrame(index=concept_df.columns[:-2], columns=['number', 'mkv'])
for col in concept_df.columns[:-2]:
    count_df.loc[col, 'number'] = concept_df[col].value_counts().loc[1]
    count_df.loc[col, 'mkv'] = concept_df.groupby(col)['TCAP'].sum().loc[1] / 1e+8
# 筛选概念
select_df = count_df[(count_df['number'] < 500) & (count_df['number'] > 40) & (count_df['mkv'] > 10000)]
big_concept = ['茅', '先进制造', '顺周期', '资源股', '新能源汽车',
               '新能源', '半导体产业', '光伏', '芯片', '锂电池',
               '白马股', '大消费', '碳中和', '元宇宙', '新基建']
# 计算重合度
sub_df = concept_df[big_concept + ['sec_code', 'TCAP']].dropna(subset=big_concept, how='all')
overlap_df = pd.DataFrame(index=big_concept, columns=big_concept)
for i in range(len(big_concept)):
    for j in range(len(big_concept)):
        if i == j:
            overlap_df.iloc[i, j] = 1.
            continue
        tmp = sub_df[[big_concept[i], big_concept[j], 'sec_code', 'TCAP']]
        tmp['sum'] = tmp[[big_concept[i], big_concept[j]]].sum(axis=1)
        overlap_cap = tmp.groupby('sum')['TCAP'].sum().loc[2.] if 2. in tmp.groupby('sum')['TCAP'].sum().index else 0
        overlap_df.iloc[i, j] = np.round(overlap_cap / tmp[tmp[big_concept[i]] == 1]['TCAP'].sum(), 2)

name_list = ['顺周期', '资源股',
             '半导体产业', '光伏', '锂电池',
             '大消费', '元宇宙', '新基建']
corr_df = overlap_df.loc[name_list, name_list]

fig = ff.create_annotated_heatmap(
    z=corr_df.to_numpy(),
    x=list(corr_df.index.values),
    y=list(corr_df.columns.values),
    xgap=3, ygap=3,
    zmin=-1, zmax=1,
    colorscale='earth',
    colorbar_thickness=30,
    colorbar_ticklen=3,
)
fig.update_layout(title_text='<b>Overlap Matrix (cont. Concept)<b>',
                  title_x=0.5,
                  titlefont={'size': 24},
                  width=800, height=600,
                  xaxis_showgrid=False,
                  xaxis={'side': 'bottom'},
                  yaxis_showgrid=False,
                  yaxis_autorange='reversed',
                  paper_bgcolor=None,
                  )
plot_ly(fig, filename="D:\\123.html", auto_open=False)
# 以因诺为例
from hbshare.quant.Kevin.quant_room.PrivateHoldingAnalysis import HoldingAnalysor
# holding_cl = HoldingAnalysor('因诺聚配中证500指数增强', "20220131", "000905")
holding_cl = HoldingAnalysor('星阔广厦1号中证500指数增强', "20220128", "000905")
portfolio_weight_series = holding_cl.data_param['portfolio_weight_series']
benchmark_weight_series = holding_cl.data_param['benchmark_weight_series']
tmp = concept_df.copy()
tmp['ticker'] = tmp['sec_code'].apply(lambda x: x.split('.')[0])
tmp = tmp.set_index('ticker')[name_list].dropna(how='all', subset=name_list)
weight_df = pd.merge(portfolio_weight_series.to_frame('pf'), benchmark_weight_series.to_frame('bm'),
                     left_index=True, right_index=True, how='outer').fillna(0.)
pf_concept = tmp.merge(weight_df, left_index=True, right_index=True)

contr_df = []
for name in name_list:
    a = pf_concept.groupby(name)[['pf', 'bm']].sum().loc[1]
    a.name = name
    contr_df.append(a)

contr_df = pd.concat(contr_df, axis=1).T


"""
可转债定价模型；B-S模型
"""
import numpy as np
import datetime as dt
import time
from scipy.stats import norm


def run_time(func):
    def call_fun(*args, **kwargs):
        start_time = time.time()
        func(*args, **kwargs)
        end_time = time.time()
        print('程序用时：%s秒' % (end_time - start_time))
    return call_fun


def _cashFlowCalc(termCoupon, strMaturityDate, strNowDate):
    """
    @param termCoupon: 票息
    @param strMaturityDate: 到期日
    @param strNowDate: 现在的日期
    """
    dtMaturityDate = dt.datetime.strptime(strMaturityDate, '%Y/%m/%d')
    dtNowDate = dt.datetime.strptime(strNowDate, '%Y/%m/%d')
    numPtmYears = (dtMaturityDate - dtNowDate).days / 365.0

    n = len(termCoupon)
    numDig, numRound = np.modf(numPtmYears)
    numPayment = int(np.min([numRound + 1, n]))

    cashFlow = termCoupon[-numPayment:]
    cashFlowTime = np.arange(numPayment) + numDig

    return cashFlow, cashFlowTime


def _pv(cashFlow, cashFlowTime, r):
    """
    @param cashFlow: 现金流
    @param cashFlowTime: 现金流对应时间
    @param r: 无风险利率
    """
    ret = 0

    for t, cash in enumerate(cashFlow):
        ret += cash / ((1 + r) ** cashFlowTime[t])

    return ret


def bondPrice(termCoupon, strMaturityDate, strNowDate, r):
    cashFlow, cashFlowTime = _cashFlowCalc(termCoupon, strMaturityDate, strNowDate)
    ret = _pv(cashFlow, cashFlowTime, r)

    return ret


def blackScholes(s, x, t, vol, r):
    """
    @param s: 标的股票的现价
    @param x: 执行价格/转股价格
    @param t: 期权到期日，按年计
    @param vol: 股票的标准差
    @param r: 无风险利率
    """
    d1 = (np.log(s / x) + (r + 0.5 * (vol ** 2)) * t) / (vol * np.sqrt(t))
    d2 = d1 - vol * np.sqrt(t)

    ret = s * norm.cdf(d1) - x * np.exp(-r * t) * norm.cdf(d2)

    return ret


def cbPricingBlackScholes(stock, term, now, vol, r, isPrint=True):
    """
    B-S定价模型
    @param stock: 当前股价
    @param term: 转债条款变量
    @param now: 当前日期
    @param vol: 隐含波动率
    @param r: 无风险利率(同等级期限信用债收益率)
    @param isPrint
    """
    straightBondValue = bondPrice(term['Coupon'], term['Maturity'], now, r)

    if isPrint:
        print("B-S模型的债底: {}".format(straightBondValue, '0.2f'))

    s = stock
    x = term['ConvPrice'] * term['Coupon'][-1] / 100.0

    dtMaturityDate = dt.datetime.strptime(term['Maturity'], '%Y/%m/%d')
    dtNowDate = dt.datetime.strptime(now, '%Y/%m/%d')

    t = (dtMaturityDate - dtNowDate).days / 365.0

    call = blackScholes(s, x, t, vol, np.log(1 + r)) * 100.0 / term['ConvPrice']

    if isPrint:
        print("B-S模型的期权部分: {}".format(call, '0.2f'))

    return straightBondValue, call


if __name__ == '__main__':
    # 山鹰转债为例
    term_CB = {
        "ConvPrice": 3.3,
        "Maturity": "2024/11/21",
        "ConvertStart": 5.5,
        "Coupon": [0.4, 0.6, 1.0, 1.5, 2.0, 113],
        "Recall": [5.5, 15, 30, 130],
        "Resell": [2, 30, 30, 70, 103]
    }

    stock_price = 3.62
    bond_value, option_value = cbPricingBlackScholes(stock_price, term_CB, '2021/9/17', 0.01, 0.0258)
    print(bond_value + option_value)


# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
from hbshare.fe.xwq.analysis.utils.const_var import TimeDateFormat
from hbshare.fe.xwq.analysis.utils.timedelta_utils import TimeDateUtil
import os
import numpy as np
import pandas as pd

from WindPy import w
w.start()  # 默认命令超时时间为120秒，如需设置超时时间可以加入waitTime参数，例如waitTime=60,即设置命令超时时间为60秒
w.isconnected()  # 判断WindPy是否已经登录成功

from matplotlib.ticker import FuncFormatter
import matplotlib
import matplotlib.pyplot as plt
import seaborn as sns
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False
sns.set_style('white', {'font.sans-serif': ['simhei', 'Arial']})
line_color_list = ['#F04950', '#6268A2', '#959595', '#333335', '#EE703F', '#7E4A9B', '#8A662C',
                   '#44488E', '#BA67E9', '#3FAEEE']
bar_color_list = ['#C94649', '#EEB2B4', '#E1777A', '#D57C56', '#E39A79', '#DB8A66', '#E5B88C',
                  '#8588B7', '#B4B6D1', '#55598D', '#628497', '#A9C6CB', '#866EA9', '#B79BC7',
                  '#7D7D7E', '#CACACA', '#A7A7A8', '#606063', '#C4C4C4', '#99999B', '#B7B7B7']


def from_rgb_to_color16(rgb):
    color = '#'
    for i in rgb:
        num = int(i)
        color += str(hex(num))[-2:].replace('x', '0').upper()
    return color


def to_100percent(temp, position):
    return '%0.01f'%(temp * 100) + '%'


def to_percent(temp, position):
    return '%0.01f'%(temp) + '%'


def filter_extreme_mad(ser, n=3):
    median = ser.quantile(0.5)
    new_median = ((ser - median).abs()).quantile(0.5)
    max_range = median + n * new_median
    min_range = median - n * new_median
    ser = np.clip(ser, min_range, max_range)
    return ser


def get_date(start_date, end_date):
    calendar_df = HBDB().read_cal(start_date, end_date)
    calendar_df = calendar_df.rename(columns={'JYRQ': 'CALENDAR_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
    calendar_df['CALENDAR_DATE'] = calendar_df['CALENDAR_DATE'].astype(str)
    calendar_df = calendar_df.sort_values('CALENDAR_DATE')
    calendar_df['IS_OPEN'] = calendar_df['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
    calendar_df['YEAR'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[:4])
    calendar_df['YEAR_MONTH'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[:6])
    calendar_df['MONTH'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:6])
    calendar_df['MONTH_DAY'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:])
    calendar_df = calendar_df[(calendar_df['CALENDAR_DATE'] >= start_date) & (calendar_df['CALENDAR_DATE'] <= end_date)]
    trade_df = calendar_df[calendar_df['IS_OPEN'] == 1].rename(columns={'CALENDAR_DATE': 'TRADE_DATE'})
    trade_df = trade_df[(trade_df['TRADE_DATE'] >= start_date) & (trade_df['TRADE_DATE'] <= end_date)]
    report_df = calendar_df.drop_duplicates('YEAR_MONTH', keep='last').rename(columns={'CALENDAR_DATE': 'REPORT_DATE'})
    report_df = report_df[report_df['MONTH_DAY'].isin(['0331', '0630', '0930', '1231'])]
    report_df = report_df[(report_df['REPORT_DATE'] >= start_date) & (report_df['REPORT_DATE'] <= end_date)]
    report_trade_df = calendar_df[calendar_df['IS_OPEN'] == 1].rename(columns={'CALENDAR_DATE': 'TRADE_DATE'})
    report_trade_df = report_trade_df.sort_values('TRADE_DATE').drop_duplicates('YEAR_MONTH', keep='last')
    report_trade_df = report_trade_df[report_trade_df['MONTH'].isin(['03', '06', '09', '12'])]
    report_trade_df = report_trade_df[(report_trade_df['TRADE_DATE'] >= start_date) & (report_trade_df['TRADE_DATE'] <= end_date)]
    calendar_trade_df = calendar_df[['CALENDAR_DATE']].merge(trade_df[['TRADE_DATE']], left_on=['CALENDAR_DATE'], right_on=['TRADE_DATE'], how='left')
    calendar_trade_df['TRADE_DATE'] = calendar_trade_df['TRADE_DATE'].fillna(method='ffill')
    calendar_trade_df = calendar_trade_df[(calendar_trade_df['TRADE_DATE'] >= start_date) & (calendar_trade_df['TRADE_DATE'] <= end_date)]
    return calendar_df, report_df, trade_df, report_trade_df, calendar_trade_df


class PeTracking:
    def __init__(self, start_date, end_date, data_path):
        self.start_date = start_date
        self.end_date = end_date
        self.start_date_hyphen = TimeDateUtil.convert_format(self.start_date, TimeDateFormat.YMD.value, TimeDateFormat.YMDHYPHEN.value)
        self.end_date_hyphen = TimeDateUtil.convert_format(self.end_date, TimeDateFormat.YMD.value, TimeDateFormat.YMDHYPHEN.value)
        self.data_path = data_path
        self.stocks = pd.read_excel('{0}stocks.xlsx'.format(self.data_path))
        self.stocks['TICKER_SYMBOL'] = self.stocks['代码'].apply(lambda x: str(x).split('.')[0])
        self.indexs = pd.read_excel('{0}indexs.xlsx'.format(self.data_path))
        self.load()

    def load(self):
        self.calendar_df, self.report_df, self.trade_df, self.report_trade_df, self.calendar_trade_df = get_date(self.start_date, self.end_date)

        if os.path.isfile('{0}stock_valuation.hdf'.format(self.data_path)):
            existed_stock_valuation = pd.read_hdf('{0}stock_valuation.hdf'.format(self.data_path), key='table')
            max_date = max(existed_stock_valuation['TRADE_DATE'])
            start_date = max(str(max_date), self.start_date)
        else:
            existed_stock_valuation = pd.DataFrame()
            start_date = self.start_date
        trade_df = self.trade_df[(self.trade_df['TRADE_DATE'] > start_date) & (self.trade_df['TRADE_DATE'] <= self.end_date)]
        stock_valuation_list = []
        for date in trade_df['TRADE_DATE'].unique().tolist():
            stock_valuation_date = HBDB().read_stock_valuation_given_date(date)
            stock_valuation_date = stock_valuation_date[['TRADE_DATE', 'TICKER_SYMBOL', 'SEC_SHORT_NAME', 'MARKET_VALUE', 'PE_TTM', 'PB_LF', 'PEG', 'DIVIDEND_RATIO_TTM', 'FORWARD_PEHR']] if len(stock_valuation_date) > 0 else pd.DataFrame(columns=['TRADE_DATE', 'TICKER_SYMBOL', 'SEC_SHORT_NAME', 'MARKET_VALUE', 'PE_TTM', 'PB_LF', 'PEG', 'DIVIDEND_RATIO_TTM', 'FORWARD_PEHR'])
            star_stock_valuation_date = HBDB().read_star_stock_valuation_given_date(date)
            star_stock_valuation_date = star_stock_valuation_date[['TRADE_DATE', 'TICKER_SYMBOL', 'SEC_SHORT_NAME', 'MARKET_VALUE', 'PE_TTM', 'PB_LF', 'PEG', 'DIVIDEND_RATIO_TTM', 'FORWARD_PEHR']] if len(star_stock_valuation_date) > 0 else pd.DataFrame(columns=['TRADE_DATE', 'TICKER_SYMBOL', 'SEC_SHORT_NAME', 'MARKET_VALUE', 'PE_TTM', 'PB_LF', 'PEG', 'DIVIDEND_RATIO_TTM', 'FORWARD_PEHR'])
            stock_valuation_date = pd.concat([stock_valuation_date, star_stock_valuation_date])
            stock_valuation_list.append(stock_valuation_date)
            print(date)
        self.stock_valuation = pd.concat([existed_stock_valuation] + stock_valuation_list, ignore_index=True)
        self.stock_valuation = self.stock_valuation.sort_values(['TRADE_DATE', 'TICKER_SYMBOL'])
        self.stock_valuation = self.stock_valuation.reset_index().drop('index', axis=1)
        self.stock_valuation.to_hdf('{0}stock_valuation.hdf'.format(self.data_path), key='table', mode='w')
        self.stock_valuation = pd.read_hdf('{0}stock_valuation.hdf'.format(self.data_path), key='table')

        if os.path.isfile('{0}index_valuation.hdf'.format(self.data_path)):
            existed_index_valuation = pd.read_hdf('{0}index_valuation.hdf'.format(self.data_path), key='table')
            max_date = max(existed_index_valuation['TRADE_DATE'])
            start_date_hyphen = max(str(max_date), self.start_date_hyphen)
        else:
            existed_index_valuation = pd.DataFrame()
            start_date_hyphen = self.start_date_hyphen
        index_valuation_list = []
        for index in self.indexs['代码'].unique().tolist():
            index_valuation_index = w.wsd(index, "pe_ttm", start_date_hyphen, self.end_date_hyphen, "Fill=Previous;PriceAdj=F", usedf=True)[1].reset_index()
            index_valuation_index.columns = ['TRADE_DATE', 'PE_TTM']
            index_valuation_index['INDEX_SYMBOL'] = index
            index_valuation_list.append(index_valuation_index)
            print(index)
        self.index_valuation = pd.concat([existed_index_valuation] + index_valuation_list, ignore_index=True)
        self.index_valuation = self.index_valuation.sort_values(['TRADE_DATE', 'INDEX_SYMBOL']).drop_duplicates(['TRADE_DATE', 'INDEX_SYMBOL'])
        self.index_valuation = self.index_valuation.reset_index().drop('index', axis=1)
        self.index_valuation.to_hdf('{0}index_valuation.hdf'.format(self.data_path), key='table', mode='w')
        self.index_valuation = pd.read_hdf('{0}index_valuation.hdf'.format(self.data_path), key='table')
        return

    def get_stocks_forwardpehr(self):
        latest_stock_valuation = self.stock_valuation[self.stock_valuation['TRADE_DATE'] == self.end_date]
        stocks_forwardpehr = self.stocks.merge(latest_stock_valuation[['TICKER_SYMBOL', 'FORWARD_PEHR']], on=['TICKER_SYMBOL'], how='left')
        stocks_forwardpehr = stocks_forwardpehr.drop('TICKER_SYMBOL', axis=1)
        stocks_forwardpehr.to_excel('{0}stocks_forwardpehr_{1}.xlsx'.format(self.data_path, self.end_date))
        return

    def get_stocks_pe_quantile(self):
        stock_valuation = self.stock_valuation[self.stock_valuation['TICKER_SYMBOL'].isin(self.stocks['TICKER_SYMBOL'].unique().tolist())]
        stock_valuation = stock_valuation.pivot(index='TRADE_DATE', columns='TICKER_SYMBOL', values='PE_TTM')
        stock_valuation = stock_valuation.sort_index()
        stocks_pe_quantile = pd.DataFrame(stock_valuation.apply(lambda x: (1.0 - np.count_nonzero(x.iloc[-1] <= x) / x.size) * 100.0))
        stocks_pe_quantile = stocks_pe_quantile.reset_index()
        stocks_pe_quantile.columns = ['TICKER_SYMBOL', 'PE_QUANTILE']
        stocks_pe_quantile = self.stocks.merge(stocks_pe_quantile, on=['TICKER_SYMBOL'], how='left')
        stocks_pe_quantile = stocks_pe_quantile.drop('TICKER_SYMBOL', axis=1)
        stocks_pe_quantile.to_excel('{0}stocks_pe_quantile_{1}.xlsx'.format(self.data_path, self.end_date))
        return

    def get_indexs_pe_quantile(self):
        index_valuation = self.index_valuation[self.index_valuation['INDEX_SYMBOL'].isin(self.indexs['代码'].unique().tolist())]
        index_valuation = index_valuation.pivot(index='TRADE_DATE', columns='INDEX_SYMBOL', values='PE_TTM')
        index_valuation = index_valuation.sort_index()
        indexs_pe_quantile = pd.DataFrame(index_valuation.apply(lambda x: (1.0 - np.count_nonzero(x.iloc[-1] <= x) / x.size) * 100.0))
        indexs_pe_quantile = indexs_pe_quantile.reset_index()
        indexs_pe_quantile.columns = ['代码', 'PE_QUANTILE']
        indexs_pe_quantile = self.indexs.merge(indexs_pe_quantile, on=['代码'], how='left')
        indexs_pe_quantile.to_excel('{0}indexs_pe_quantile_{1}.xlsx'.format(self.data_path, self.end_date))

        for sector in self.indexs['板块'].unique().tolist():
            sector_indexs = self.indexs[self.indexs['板块'] == sector]['代码'].unique().tolist()
            index_name_list = self.indexs[self.indexs['板块'] == sector]['名称'].unique().tolist()
            index_pe_list = [filter_extreme_mad(index_valuation[index].dropna()) for index in sector_indexs]
            ind_pe_latest = index_valuation[sector_indexs].iloc[-1]
            plt.figure(figsize=(12, 6))
            plt.boxplot(index_pe_list, labels=index_name_list, vert=True, widths=0.25, flierprops={'marker': 'o', 'markersize': 1}, meanline=True, showmeans=True, showfliers=False)
            plt.scatter(range(1, len(ind_pe_latest) + 1), ind_pe_latest.values, marker='o')
            plt.xticks(rotation=45)
            plt.title(sector)
            plt.tight_layout()
            plt.savefig('{0}indexs_pe_quantile_{1}_{2}.png'.format(self.data_path, sector, self.end_date))
        return

    def get_amt_results(self):
        self.get_stocks_forwardpehr()
        self.get_stocks_pe_quantile()
        self.get_indexs_pe_quantile()


class IndustryTracking:
    def __init__(self, start_date, end_date, data_path):
        self.start_date = start_date
        self.end_date = end_date
        self.start_date_hyphen = TimeDateUtil.convert_format(self.start_date, TimeDateFormat.YMD.value, TimeDateFormat.YMDHYPHEN.value)
        self.end_date_hyphen = TimeDateUtil.convert_format(self.end_date, TimeDateFormat.YMD.value, TimeDateFormat.YMDHYPHEN.value)
        self.data_path = data_path
        self.stocks = pd.read_excel('{0}stocks.xlsx'.format(self.data_path))
        self.stocks['TICKER_SYMBOL'] = self.stocks['代码'].apply(lambda x: str(x).split('.')[0])
        self.indexs = pd.read_excel('{0}indexs_factor_ori.xlsx'.format(self.data_path))
        self.indexs = self.indexs[self.indexs['是否展示'] == 1]
        self.load()

    def load(self):
        self.calendar_df, self.report_df, self.trade_df, self.report_trade_df, self.calendar_trade_df = get_date(self.start_date, self.end_date)

        if os.path.isfile('{0}stock_valuation.hdf'.format(self.data_path)):
            existed_stock_valuation = pd.read_hdf('{0}stock_valuation.hdf'.format(self.data_path), key='table')
            max_date = max(existed_stock_valuation['TRADE_DATE'])
            start_date = max(str(max_date), self.start_date)
        else:
            existed_stock_valuation = pd.DataFrame()
            start_date = self.start_date
        trade_df = self.trade_df[(self.trade_df['TRADE_DATE'] > start_date) & (self.trade_df['TRADE_DATE'] <= self.end_date)]
        stock_valuation_list = []
        for date in trade_df['TRADE_DATE'].unique().tolist():
            stock_valuation_date = HBDB().read_stock_valuation_given_date(date)
            stock_valuation_date = stock_valuation_date[['TRADE_DATE', 'TICKER_SYMBOL', 'SEC_SHORT_NAME', 'MARKET_VALUE', 'PE_TTM', 'PB_LF', 'PEG', 'DIVIDEND_RATIO_TTM', 'FORWARD_PEHR']] if len(stock_valuation_date) > 0 else pd.DataFrame(columns=['TRADE_DATE', 'TICKER_SYMBOL', 'SEC_SHORT_NAME', 'MARKET_VALUE', 'PE_TTM', 'PB_LF', 'PEG', 'DIVIDEND_RATIO_TTM', 'FORWARD_PEHR'])
            star_stock_valuation_date = HBDB().read_star_stock_valuation_given_date(date)
            star_stock_valuation_date = star_stock_valuation_date[['TRADE_DATE', 'TICKER_SYMBOL', 'SEC_SHORT_NAME', 'MARKET_VALUE', 'PE_TTM', 'PB_LF', 'PEG', 'DIVIDEND_RATIO_TTM', 'FORWARD_PEHR']] if len(star_stock_valuation_date) > 0 else pd.DataFrame(columns=['TRADE_DATE', 'TICKER_SYMBOL', 'SEC_SHORT_NAME', 'MARKET_VALUE', 'PE_TTM', 'PB_LF', 'PEG', 'DIVIDEND_RATIO_TTM', 'FORWARD_PEHR'])
            stock_valuation_date = pd.concat([stock_valuation_date, star_stock_valuation_date])
            stock_valuation_list.append(stock_valuation_date)
            print(date)
        self.stock_valuation = pd.concat([existed_stock_valuation] + stock_valuation_list, ignore_index=True)
        self.stock_valuation = self.stock_valuation.sort_values(['TRADE_DATE', 'TICKER_SYMBOL'])
        self.stock_valuation = self.stock_valuation.reset_index().drop('index', axis=1)
        self.stock_valuation.to_hdf('{0}stock_valuation.hdf'.format(self.data_path), key='table', mode='w')
        self.stock_valuation = pd.read_hdf('{0}stock_valuation.hdf'.format(self.data_path), key='table')

        if os.path.isfile('{0}index_valuation.hdf'.format(self.data_path)):
            existed_index_valuation = pd.read_hdf('{0}index_valuation.hdf'.format(self.data_path), key='table')
            max_date = max(existed_index_valuation['TRADE_DATE'])
            start_date_hyphen = max(str(max_date), self.start_date_hyphen)
        else:
            existed_index_valuation = pd.DataFrame()
            start_date_hyphen = self.start_date_hyphen
        index_valuation_list = []
        for index in self.indexs['代码'].unique().tolist():
            index_valuation_index = w.wsd(index, "pe_ttm", start_date_hyphen, self.end_date_hyphen, "Fill=Previous;PriceAdj=F", usedf=True)[1].reset_index()
            index_valuation_index.columns = ['TRADE_DATE', 'PE_TTM']
            index_valuation_index['INDEX_SYMBOL'] = index
            index_valuation_list.append(index_valuation_index)
            print(index)
        self.index_valuation = pd.concat([existed_index_valuation] + index_valuation_list, ignore_index=True)
        self.index_valuation = self.index_valuation.sort_values(['TRADE_DATE', 'INDEX_SYMBOL']).drop_duplicates(['TRADE_DATE', 'INDEX_SYMBOL'])
        self.index_valuation = self.index_valuation.reset_index().drop('index', axis=1)
        self.index_valuation.to_hdf('{0}index_valuation.hdf'.format(self.data_path), key='table', mode='w')
        self.index_valuation = pd.read_hdf('{0}index_valuation.hdf'.format(self.data_path), key='table')

        if os.path.isfile('{0}index_daily_k.hdf'.format(self.data_path)):
            existed_index_daily_K = pd.read_hdf('{0}index_daily_k.hdf'.format(self.data_path), key='table')
            max_date = max(existed_index_daily_K['TRADE_DATE'])
            start_date_hyphen = max(str(max_date), self.start_date_hyphen)
        else:
            existed_index_daily_K = pd.DataFrame()
            start_date_hyphen = self.start_date_hyphen
        index_daily_k_list = []
        for index in self.indexs['代码'].unique().tolist():
            index_daily_k_index = w.wsd(index, "close", start_date_hyphen, self.end_date_hyphen, "Fill=Previous;PriceAdj=F", usedf=True)[1].reset_index()
            index_daily_k_index.columns = ['TRADE_DATE', 'CLOSE_INDEX']
            index_daily_k_index['INDEX_SYMBOL'] = index
            index_daily_k_list.append(index_daily_k_index)
            print(index)
        self.index_daily_k = pd.concat([existed_index_daily_K] + index_daily_k_list, ignore_index=True)
        self.index_daily_k = self.index_daily_k.sort_values(['TRADE_DATE', 'INDEX_SYMBOL']).drop_duplicates(['TRADE_DATE', 'INDEX_SYMBOL'])
        self.index_daily_k = self.index_daily_k.reset_index().drop('index', axis=1)
        self.index_daily_k.to_hdf('{0}index_daily_k.hdf'.format(self.data_path), key='table', mode='w')
        self.index_daily_k = pd.read_hdf('{0}index_daily_k.hdf'.format(self.data_path), key='table')

        start_date_hyphen = str(int(self.start_date_hyphen[:4]) - 1) + self.start_date_hyphen[4:]
        index_finance_list = []
        for index in self.indexs['代码'].unique().tolist():
            index_finance_index = w.wsd(index, "oper_rev,or_ttm2,yoy_or,qfa_yoysales,qfa_cgrsales,np_belongto_parcomsh,profit_ttm2,yoynetprofit,qfa_yoynetprofit,qfa_cgrnetprofit,roe_avg,qfa_roe,yoyroe,roa,qfa_roa,grossprofitmargin,qfa_grossprofitmargin,netprofitmargin,qfa_netprofitmargin,eps_basic,bps", start_date_hyphen, self.end_date_hyphen, "unit=1;rptType=1;Period=Q;Days=Alldays", usedf=True)[1].reset_index()
            index_finance_index.columns = ['REPORT_DATE', 'MAIN_INCOME', 'MAIN_INCOME_TTM', 'MAIN_INCOME_YOY', 'MAIN_INCOME_Q_YOY', 'MAIN_INCOME_Q_MOM', 'NET_PROFIT', 'PROFIT_TTM', 'NET_PROFIT_YOY', 'NET_PROFIT_Q_YOY', 'NET_PROFIT_Q_MOM', 'ROE', 'ROE_Q', 'ROE_YOY', 'ROA', 'ROA_Q', 'GROSS_PROFIT_MARGIN', 'GROSS_PROFIT_MARGIN_Q', 'NET_PROFIT_MARGIN', 'NET_PROFIT_MARGIN_Q', 'EPS_BASIC', 'BPS']
            index_finance_index['INDEX_SYMBOL'] = index
            index_finance_list.append(index_finance_index)
            print(index)
        self.index_finance = pd.concat(index_finance_list, ignore_index=True)
        self.index_finance = self.index_finance.sort_values(['REPORT_DATE', 'INDEX_SYMBOL']).drop_duplicates(['REPORT_DATE', 'INDEX_SYMBOL'])
        self.index_finance = self.index_finance.reset_index().drop('index', axis=1)
        self.index_finance.to_hdf('{0}index_finance.hdf'.format(self.data_path), key='table', mode='w')
        self.index_finance = pd.read_hdf('{0}index_finance.hdf'.format(self.data_path), key='table')

        start_date_hyphen = str(int(self.start_date_hyphen[:4]) - 1) + self.start_date_hyphen[4:]
        index_consensus_list = []
        for index in self.indexs['代码'].unique().tolist():
            index_consensus_index = w.wsd(index, "west_sales_FY1,west_sales_YOY,west_sales_CAGR,west_netprofit_FY1,west_netprofit_YOY,west_netprofit_CAGR,west_nproc_1w,west_nproc_4w,west_nproc_13w,west_nproc_26w,west_avgroe_FY1,west_avgroe_YOY,west_eps_FY1,west_avgbps_FY1", start_date_hyphen, self.end_date_hyphen, "unit=1;year=2022;Period=Q;Days=Alldays", usedf=True)[1].reset_index()
            index_consensus_index.columns = ['REPORT_DATE', 'CON_MAIN_INCOME_FY1', 'CON_MAIN_INCOME_YOY', 'CON_MAIN_INCOME_CAGR', 'CON_NET_PROFIT_FY1', 'CON_NET_PROFIT_YOY', 'CON_NET_PROFIT_CAGR', 'CON_NET_PROFIT_1W', 'CON_NET_PROFIT_4W', 'CON_NET_PROFIT_13W', 'CON_NET_PROFIT_26W', 'CON_ROE_FY1', 'CON_ROE_YOY', 'CON_EPS_FY1', 'CON_BPS_FY1']
            index_consensus_index['INDEX_SYMBOL'] = index
            index_consensus_list.append(index_consensus_index)
            print(index)
        self.index_consensus = pd.concat(index_consensus_list, ignore_index=True)
        self.index_consensus = self.index_consensus.sort_values(['REPORT_DATE', 'INDEX_SYMBOL']).drop_duplicates(['REPORT_DATE', 'INDEX_SYMBOL'])
        self.index_consensus = self.index_consensus.reset_index().drop('index', axis=1)
        self.index_consensus.to_hdf('{0}index_consensus.hdf'.format(self.data_path), key='table', mode='w')
        self.index_consensus = pd.read_hdf('{0}index_consensus.hdf'.format(self.data_path), key='table')

        # if os.path.isfile('{0}index_consensus.hdf'.format(self.data_path)):
        #     existed_index_consensus = pd.read_hdf('{0}index_consensus.hdf'.format(self.data_path), key='table')
        #     max_date = max(existed_index_consensus['TRADE_DATE'])
        #     start_date_hyphen = max(str(max_date), self.start_date_hyphen)
        # else:
        #     existed_index_consensus = pd.DataFrame()
        #     start_date_hyphen = self.start_date_hyphen
        # index_consensus_list = []
        # for index in self.indexs['代码'].unique().tolist()[:3]:
        #     index_consensus_index = w.wsd(index, "west_sales_FY1,west_sales_YOY,west_sales_CAGR,west_netprofit_FY1,west_netprofit_YOY,west_netprofit_CAGR,west_nproc_1w,west_nproc_4w,west_nproc_13w,west_nproc_26w,west_avgroe_FY1,west_avgroe_YOY,west_eps_FY1,west_avgbps_FY1", start_date_hyphen, self.end_date_hyphen, "unit=1;year=2022;Days=Alldays;Fill=Previous", usedf=True)[1].reset_index()
        #     index_consensus_index.columns = ['TRADE_DATE', 'CON_MAIN_INCOME_FY1', 'CON_MAIN_INCOME_YOY', 'CON_MAIN_INCOME_CAGR', 'CON_NET_PROFIT_FY1', 'CON_NET_PROFIT_YOY', 'CON_NET_PROFIT_CAGR', 'CON_NET_PROFIT_1W', 'CON_NET_PROFIT_4W', 'CON_NET_PROFIT_13W', 'CON_NET_PROFIT_26W', 'CON_ROE_FY1', 'CON_ROE_YOY', 'CON_EPS_FY1', 'CON_BPS_FY1']
        #     index_consensus_index['INDEX_SYMBOL'] = index
        #     index_consensus_list.append(index_consensus_index)
        #     print(index)
        # self.index_consensus = pd.concat([existed_index_consensus] + index_consensus_list, ignore_index=True)
        # self.index_consensus = self.index_consensus.sort_values(['TRADE_DATE', 'INDEX_SYMBOL']).drop_duplicates(['TRADE_DATE', 'INDEX_SYMBOL'])
        # self.index_consensus = self.index_consensus.reset_index().drop('index', axis=1)
        # self.index_consensus.to_hdf('{0}index_consensus.hdf'.format(self.data_path), key='table', mode='w')
        # self.index_consensus = pd.read_hdf('{0}index_consensus.hdf'.format(self.data_path), key='table')
        return

    def get_amt_ret(self):
        index_daily_k = self.index_daily_k.pivot(index='TRADE_DATE', columns='INDEX_SYMBOL', values='CLOSE_INDEX')
        index_daily_k.index = map(lambda x: str(x).replace('-', ''), index_daily_k.index)
        index_daily_k = index_daily_k.sort_index()

        # 周收益情况
        last_date = self.trade_df[(self.trade_df['IS_WEEK_END'] == '1') & (self.trade_df['TRADE_DATE'] <= self.end_date)]['TRADE_DATE'].iloc[-2]
        index_daily_k_w = index_daily_k[(index_daily_k.index >= last_date) & (index_daily_k.index <= self.end_date)]
        index_daily_k_wret = index_daily_k_w.iloc[-1] / index_daily_k_w.iloc[0] - 1
        index_daily_k_wret = index_daily_k_wret.reset_index()
        index_daily_k_wret.columns = ['代码', '周收益']
        indexs_wret = self.indexs.merge(index_daily_k_wret, on=['代码'], how='left')
        sector_list = self.indexs['板块'].unique().tolist()
        sector_color_list = ([bar_color_list[0], bar_color_list[7]] * len(sector_list))[:len(sector_list)]
        sector_color_dict = {sector: sector_color_list[i] for i, sector in enumerate(sector_list)}
        indexs_wret['颜色'] = indexs_wret['板块'].apply(lambda x: sector_color_dict[x])
        indexs_wret['索引'] = range(len(indexs_wret))
        # 周收益柱状图
        fig, ax = plt.subplots(figsize=(12, 6))
        sns.barplot(x='索引', y='周收益', data=indexs_wret, palette=indexs_wret['颜色'].tolist())
        ax.set_xticklabels(labels=indexs_wret['展示名称'].tolist(), rotation=90)
        ax.yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.xlabel('')
        plt.ylabel('周收益')
        plt.tight_layout()
        plt.savefig('{0}wret_{1}.png'.format(self.data_path, self.end_date))

        # 先进制造情况
        amt_indexs_names_dic = self.indexs[self.indexs['板块'] == '先进制造'].set_index('代码')['展示名称'].to_dict()
        amt_indexs = self.indexs[self.indexs['板块'] == '先进制造']['代码'].unique().tolist()
        amt_index_daily_k = index_daily_k[amt_indexs]
        amt_index_daily_k = amt_index_daily_k / amt_index_daily_k.iloc[0]
        amt_index_daily_k.columns = [amt_indexs_names_dic[col] for col in list(amt_index_daily_k.columns)]
        amt_index_daily_k = amt_index_daily_k.reset_index().rename(columns={'index': 'TRADE_DATE'})
        amt_index_daily_k['TRADE_DATE'] = amt_index_daily_k['TRADE_DATE'].apply(lambda x: TimeDateUtil.convert_str_to_date(x, '%Y%m%d'))
        # 指数走势线型图
        plt.figure(figsize=(12, 6))
        for i, index_name in enumerate(list(amt_index_daily_k.columns)[1:]):
            sns.lineplot(x='TRADE_DATE', y=index_name, data=amt_index_daily_k, label=index_name, color=line_color_list[i])
        plt.xlabel('')
        plt.ylabel('')
        plt.legend(loc=2)
        plt.tight_layout()
        plt.savefig('{0}close_{1}.png'.format(self.data_path, self.end_date))
        return

    def get_amt_valuation(self):
        index_valuation = self.index_valuation.pivot(index='TRADE_DATE', columns='INDEX_SYMBOL', values='PE_TTM')
        index_valuation.index = map(lambda x: str(x).replace('-', ''), index_valuation.index)
        index_valuation = index_valuation.sort_index()

        # 估值分布情况
        indexs_info = self.indexs.copy(deep=True)
        index_id_list = indexs_info['代码'].tolist()[::-1]
        index_name_list = indexs_info['展示名称'].tolist()[::-1]
        index_valuation_list = [filter_extreme_mad(index_valuation[index].dropna()) for index in index_id_list]
        index_valuation_latest = index_valuation[index_id_list].iloc[-1]
        sector_list = indexs_info['板块'].unique().tolist()
        sector_color_list = ([bar_color_list[0], bar_color_list[7]] * len(sector_list))[:len(sector_list)]
        sector_color_dict = {sector: sector_color_list[i] for i, sector in enumerate(sector_list)}
        indexs_info['颜色'] = indexs_info['板块'].apply(lambda x: sector_color_dict[x])
        # 估值分布箱型图
        plt.figure(figsize=(12, 18))
        f = plt.boxplot(index_valuation_list, labels=index_name_list, vert=False, meanline=True, showmeans=True, showfliers=False, patch_artist=True, zorder=1)
        for box, c in zip(f['boxes'], indexs_info['颜色'].tolist()[::-1]):
            box.set(color=c)
        plt.scatter(index_valuation_latest.values, range(1, len(index_valuation_latest) + 1), marker='o', zorder=2)
        plt.xlabel('PE_TTM')
        plt.xticks(rotation=90)
        plt.tight_layout()
        plt.savefig('{0}valuation_{1}.png'.format(self.data_path, self.end_date))
        return

    def get_amt_prosperity(self):
        indexs = self.indexs.copy(deep=True)
        indexs_names_dic = indexs.set_index('代码')['展示名称'].to_dict()
        index_finance = self.index_finance.copy(deep=True)
        index_finance['REPORT_DATE'] = index_finance['REPORT_DATE'].apply(lambda x: str(x).replace('-', ''))
        index_finance = index_finance[index_finance['REPORT_DATE'] >= '20190101']
        index_daily_k = self.index_daily_k.copy(deep=True)
        index_daily_k['TRADE_DATE'] = index_daily_k['TRADE_DATE'].apply(lambda x: str(x).replace('-', ''))

        # 测试指标有效性
        factor_list = ['MAIN_INCOME', 'MAIN_INCOME_TTM', 'MAIN_INCOME_YOY', 'MAIN_INCOME_Q_YOY', 'MAIN_INCOME_Q_MOM', 'NET_PROFIT', 'PROFIT_TTM', 'NET_PROFIT_YOY', 'NET_PROFIT_Q_YOY', 'NET_PROFIT_Q_MOM', 'ROE', 'ROE_Q', 'ROE_YOY', 'ROA', 'ROA_Q', 'GROSS_PROFIT_MARGIN', 'GROSS_PROFIT_MARGIN_Q', 'NET_PROFIT_MARGIN', 'NET_PROFIT_MARGIN_Q', 'EPS_BASIC', 'BPS']
        factor_name_list = ['营业收入（元）', '营业收入TTM（元）', '营业收入同比增长率（%）', '单季度营业收入同比增长率（%）', '单季度营业收入环比增长率（%）', '归母净利润（元）', '净利润TTM（元）', '归母净利润同比增长率（%）', '单季度归母净利润同比增长率（%）', '单季度归母净利润环比增长率（%）', 'ROE', '单季度ROE', 'ROE同比增长率（%）', 'ROA', '单季度ROA', '销售毛利率（%）', '单季度销售毛利率（%）', '销售净利率（%）', '单季度销售净利率（%）', 'EPS（基本）', 'BPS']
        factor_name_dict = {factor_list[i]: factor_name_list[i] for i in range(len(factor_list))}
        corr_df = pd.DataFrame(index=self.indexs['代码'].unique().tolist(), columns=factor_list)
        for factor in factor_list:
            for index in self.indexs['代码'].unique().tolist():
                index_daily_k_index = index_daily_k[index_daily_k['INDEX_SYMBOL'] == index]
                index_finance_index = index_finance[index_finance['INDEX_SYMBOL'] == index]
                max_report_date = index_finance_index.dropna(subset=['MAIN_INCOME'])['REPORT_DATE'].max()
                index_finance_index = index_finance_index[index_finance_index['REPORT_DATE'] <= max_report_date]
                index_finance_index = index_finance_index[['REPORT_DATE', 'INDEX_SYMBOL', factor]]
                index_finance_index = index_finance_index.merge(self.calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                index_finance_index = index_finance_index.merge(index_daily_k_index, on=['INDEX_SYMBOL', 'TRADE_DATE'], how='left')
                # index_finance_index['CLOSE_INDEX'] = index_finance_index['CLOSE_INDEX'].pct_change().shift(-1)
                index_finance_index = index_finance_index.dropna().drop('TRADE_DATE', axis=1).sort_values('REPORT_DATE')
                if len(index_finance_index) == 0:
                    continue
                corr = round(np.corrcoef(index_finance_index[factor], index_finance_index['CLOSE_INDEX'])[0, 1], 2)
                corr_df.loc[index, factor] = corr

                # fig, ax1 = plt.subplots(figsize=(6, 3))
                # ax2 = ax1.twinx()
                # sns.lineplot(ax=ax1, x='REPORT_DATE', y=factor, data=index_finance_index, color=line_color_list[0])
                # sns.lineplot(ax=ax2, x='REPORT_DATE', y='CLOSE_INDEX', data=index_finance_index, color=line_color_list[1])
                # ax1.set_xlabel('')
                # ax2.set_xlabel('')
                # ax1.set_ylabel(factor_name_dict[factor])
                # ax2.set_ylabel(indexs_names_dic[index])
                # ax1.set_xticklabels(labels=index_finance_index['REPORT_DATE'].unique().tolist(), rotation=90)
                # ax2.set_xticklabels(labels=index_finance_index['REPORT_DATE'].unique().tolist(), rotation=90)
                # plt.title('两者相关系数为{2}%'.format(factor_name_dict[factor], indexs_names_dic[index], corr))
                # plt.tight_layout()
                # plt.savefig('{0}corr/{1}_{2}_{3}.png'.format(self.data_path, factor_name_dict[factor], indexs_names_dic[index], self.end_date))

        corr_df = corr_df.reset_index()
        corr_df.columns = ['代码'] + [factor_name_dict[col] for col in list(corr_df.columns)[1:]]
        indexs_corr = self.indexs.merge(corr_df, on=['代码'], how='left')
        indexs_corr.to_excel('{0}factor_close_corr_{1}.xlsx'.format(self.data_path, self.end_date))
        # indexs_corr.to_excel('{0}factor_ret_corr_{1}.xlsx'.format(self.data_path, self.end_date))
        # indexs_corr.to_excel('{0}factor_forward_ret_corr_{1}.xlsx'.format(self.data_path, self.end_date))
        return

    def get_amt_con_prosperity(self):
        indexs = self.indexs.copy(deep=True)
        indexs_names_dic = indexs.set_index('代码')['展示名称'].to_dict()
        index_consensus = self.index_consensus.copy(deep=True)
        index_consensus['REPORT_DATE'] = index_consensus['REPORT_DATE'].apply(lambda x: str(x).replace('-', ''))
        index_consensus = index_consensus[index_consensus['REPORT_DATE'] >= '20190101']
        index_finance = self.index_finance.copy(deep=True)
        index_finance['REPORT_DATE'] = index_finance['REPORT_DATE'].apply(lambda x: str(x).replace('-', ''))
        index_finance = index_finance[index_finance['REPORT_DATE'] >= '20190101']
        index_daily_k = self.index_daily_k.copy(deep=True)
        index_daily_k['TRADE_DATE'] = index_daily_k['TRADE_DATE'].apply(lambda x: str(x).replace('-', ''))

        # 测试指标有效性
        factor_list = ['CON_MAIN_INCOME_FY1', 'CON_MAIN_INCOME_YOY', 'CON_MAIN_INCOME_CAGR', 'CON_NET_PROFIT_FY1', 'CON_NET_PROFIT_YOY', 'CON_NET_PROFIT_CAGR', 'CON_NET_PROFIT_1W', 'CON_NET_PROFIT_4W', 'CON_NET_PROFIT_13W', 'CON_NET_PROFIT_26W', 'CON_ROE_FY1', 'CON_ROE_YOY', 'CON_EPS_FY1', 'CON_BPS_FY1']
        factor_name_list = ['一致预测营业收入（元）', '一致预测营业收入同比增长率（%）', '一致预测营业收入2年复合增长率（%）', '一致预测净利润（元）', '一致预测净利润同比增长率（%）', '一致预测净利润2年复合增长率（%）', '一致预测净利润1周变化率（%）', '一致预测净利润4周变化率（%）', '一致预测净利润13周变化率（%）', '一致预测净利润26周变化率（%）', '一致预测ROE（%）', '一致预测ROE同比增长率（%）', '一致预测EPS', '一致预测BPS']
        factor_name_dict = {factor_list[i]: factor_name_list[i] for i in range(len(factor_list))}
        corr_df = pd.DataFrame(index=self.indexs['代码'].unique().tolist(), columns=factor_list)
        for factor in factor_list:
            for index in self.indexs['代码'].unique().tolist():
                index_daily_k_index = index_daily_k[index_daily_k['INDEX_SYMBOL'] == index]
                index_consensus_index = index_consensus[index_consensus['INDEX_SYMBOL'] == index]
                index_finance_index = index_finance[index_finance['INDEX_SYMBOL'] == index]
                max_report_date = index_finance_index.dropna(subset=['MAIN_INCOME'])['REPORT_DATE'].max()
                index_consensus_index = index_consensus_index[index_consensus_index['REPORT_DATE'] <= max_report_date]
                index_consensus_index = index_consensus_index[['REPORT_DATE', 'INDEX_SYMBOL', factor]]
                index_consensus_index = index_consensus_index.merge(self.calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
                index_consensus_index = index_consensus_index.merge(index_daily_k_index, on=['INDEX_SYMBOL', 'TRADE_DATE'], how='left')
                # index_consensus_index['CLOSE_INDEX'] = index_consensus_index['CLOSE_INDEX'].pct_change().shift(-1)
                index_consensus_index = index_consensus_index.dropna().drop('TRADE_DATE', axis=1).sort_values('REPORT_DATE')
                if len(index_consensus_index) == 0:
                    continue
                corr = round(np.corrcoef(index_consensus_index[factor], index_consensus_index['CLOSE_INDEX'])[0, 1], 2)
                corr_df.loc[index, factor] = corr

                # fig, ax1 = plt.subplots(figsize=(6, 3))
                # ax2 = ax1.twinx()
                # sns.lineplot(ax=ax1, x='REPORT_DATE', y=factor, data=index_consensus_index, color=line_color_list[0])
                # sns.lineplot(ax=ax2, x='REPORT_DATE', y='CLOSE_INDEX', data=index_consensus_index, color=line_color_list[1])
                # ax1.set_xlabel('')
                # ax2.set_xlabel('')
                # ax1.set_ylabel(factor_name_dict[factor])
                # ax2.set_ylabel(indexs_names_dic[index])
                # ax1.set_xticklabels(labels=index_consensus_index['REPORT_DATE'].unique().tolist(), rotation=90)
                # ax2.set_xticklabels(labels=index_consensus_index['REPORT_DATE'].unique().tolist(), rotation=90)
                # plt.title('两者相关系数为{2}%'.format(factor_name_dict[factor], indexs_names_dic[index], corr))
                # plt.tight_layout()
                # plt.savefig('{0}con_corr/{1}_{2}_{3}.png'.format(self.data_path, factor_name_dict[factor], indexs_names_dic[index], self.end_date))

        corr_df = corr_df.reset_index()
        corr_df.columns = ['代码'] + [factor_name_dict[col] for col in list(corr_df.columns)[1:]]
        indexs_corr = self.indexs.merge(corr_df, on=['代码'], how='left')
        indexs_corr.to_excel('{0}con_factor_close_corr_{1}.xlsx'.format(self.data_path, self.end_date))
        # indexs_corr.to_excel('{0}con_factor_ret_corr_{1}.xlsx'.format(self.data_path, self.end_date))
        # indexs_corr.to_excel('{0}con_factor_forward_ret_corr_{1}.xlsx'.format(self.data_path, self.end_date))
        return

    def get_amt_prosperity_factor(self):
        factor_list = []
        for file_name in ['factor_close_corr', 'factor_ret_corr', 'con_factor_close_corr', 'con_factor_ret_corr']:
            if file_name in ['factor_close_corr', 'factor_ret_corr']:
                factor_name_list = ['营业收入（元）', '营业收入TTM（元）', '营业收入同比增长率（%）', '单季度营业收入同比增长率（%）', '单季度营业收入环比增长率（%）', '归母净利润（元）', '净利润TTM（元）', '归母净利润同比增长率（%）', '单季度归母净利润同比增长率（%）', '单季度归母净利润环比增长率（%）', 'ROE', '单季度ROE', 'ROE同比增长率（%）', 'ROA', '单季度ROA', '销售毛利率（%）', '单季度销售毛利率（%）', '销售净利率（%）', '单季度销售净利率（%）', 'EPS（基本）', 'BPS']
            else:
                factor_name_list = ['一致预测营业收入（元）', '一致预测营业收入同比增长率（%）', '一致预测营业收入2年复合增长率（%）', '一致预测净利润（元）', '一致预测净利润同比增长率（%）', '一致预测净利润2年复合增长率（%）', '一致预测净利润1周变化率（%）', '一致预测净利润4周变化率（%）', '一致预测净利润13周变化率（%）', '一致预测净利润26周变化率（%）', '一致预测ROE（%）', '一致预测ROE同比增长率（%）', '一致预测EPS', '一致预测BPS']
            indexs_corr_factor = pd.read_excel('{0}{1}_{2}.xlsx'.format(self.data_path, file_name, self.end_date))
            indexs_corr_factor = indexs_corr_factor[['代码'] + factor_name_list].set_index('代码')
            indexs_corr_factor = indexs_corr_factor.fillna(-99999.0)
            indexs_corr_factor = indexs_corr_factor > 0.5
            indexs_corr_factor = indexs_corr_factor.unstack().reset_index()
            indexs_corr_factor.columns = ['景气度指标', '代码', '是否有效']
            indexs_corr_factor = indexs_corr_factor[indexs_corr_factor['是否有效'] == True]
            factor_list.append(indexs_corr_factor)
        factor = pd.concat(factor_list).drop_duplicates()
        factor = factor.groupby('代码').apply(lambda x: '，'.join(x['景气度指标'])).reset_index().rename(columns={0: '景气度指标'})

        con_factor_list = []
        for file_name in ['factor_forward_ret_corr', 'con_factor_forward_ret_corr']:
            if file_name in ['factor_forward_ret_corr']:
                factor_name_list = ['营业收入（元）', '营业收入TTM（元）', '营业收入同比增长率（%）', '单季度营业收入同比增长率（%）', '单季度营业收入环比增长率（%）', '归母净利润（元）', '净利润TTM（元）', '归母净利润同比增长率（%）', '单季度归母净利润同比增长率（%）', '单季度归母净利润环比增长率（%）', 'ROE', '单季度ROE', 'ROE同比增长率（%）', 'ROA', '单季度ROA', '销售毛利率（%）', '单季度销售毛利率（%）', '销售净利率（%）', '单季度销售净利率（%）', 'EPS（基本）', 'BPS']
            else:
                factor_name_list = ['一致预测营业收入（元）', '一致预测营业收入同比增长率（%）', '一致预测营业收入2年复合增长率（%）', '一致预测净利润（元）', '一致预测净利润同比增长率（%）', '一致预测净利润2年复合增长率（%）', '一致预测净利润1周变化率（%）', '一致预测净利润4周变化率（%）', '一致预测净利润13周变化率（%）', '一致预测净利润26周变化率（%）', '一致预测ROE（%）', '一致预测ROE同比增长率（%）', '一致预测EPS', '一致预测BPS']
            indexs_corr_con_factor = pd.read_excel('{0}{1}_{2}.xlsx'.format(self.data_path, file_name, self.end_date))
            indexs_corr_con_factor = indexs_corr_con_factor[['代码'] + factor_name_list].set_index('代码')
            indexs_corr_con_factor = indexs_corr_con_factor.fillna(-99999.0)
            indexs_corr_con_factor = indexs_corr_con_factor > 0.5
            indexs_corr_con_factor = indexs_corr_con_factor.unstack().reset_index()
            indexs_corr_con_factor.columns = ['未来景气度指标', '代码', '是否有效']
            indexs_corr_con_factor = indexs_corr_con_factor[indexs_corr_con_factor['是否有效'] == True]
            con_factor_list.append(indexs_corr_con_factor)
        con_factor = pd.concat(con_factor_list).drop_duplicates()
        con_factor = con_factor.groupby('代码').apply(lambda x: '，'.join(x['未来景气度指标'])).reset_index().rename(columns={0: '未来景气度指标'})

        indexs_factor = pd.read_excel('{0}indexs_factor_ori.xlsx'.format(self.data_path))
        indexs_factor = indexs_factor.merge(factor, on=['代码'], how='left').merge(con_factor, on=['代码'], how='left')
        indexs_factor.to_excel('{0}indexs_factor.xlsx'.format(self.data_path))
        return

    def get_amt_pb_roe(self):
        indexs = self.indexs.copy(deep=True)
        indexs_names_dic = indexs.set_index('代码')['展示名称'].to_dict()
        index_daily_k = self.index_daily_k.copy(deep=True)
        index_daily_k['TRADE_DATE'] = index_daily_k['TRADE_DATE'].apply(lambda x: str(x).replace('-', ''))
        index_daily_k = index_daily_k[index_daily_k['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        index_daily_k = index_daily_k[(index_daily_k['TRADE_DATE'] >= self.start_date) & (index_daily_k['TRADE_DATE'] <= self.end_date)]
        index_finance = self.index_finance.copy(deep=True)
        index_finance['REPORT_DATE'] = index_finance['REPORT_DATE'].apply(lambda x: str(x).replace('-', ''))
        index_finance = index_finance[index_finance['REPORT_DATE'] >= self.start_date]
        index_consensus = self.index_consensus.copy(deep=True)
        index_consensus['REPORT_DATE'] = index_consensus['REPORT_DATE'].apply(lambda x: str(x).replace('-', ''))
        index_consensus = index_consensus[index_consensus['REPORT_DATE'] >= self.start_date]
        index_con_roe = pd.read_excel('{0}index_con_roe.xlsx'.format(data_path))
        index_con_roe = index_con_roe.set_index('DateTime').unstack().reset_index()
        index_con_roe.columns = ['INDEX_SYMBOL', 'TRADE_DATE', 'CON_ROE_FY1']
        index_con_roe['TRADE_DATE'] = index_con_roe['TRADE_DATE'].apply(lambda x: str(x).replace('-', '')[:8])
        index_con_roe = index_con_roe[index_con_roe['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        index_con_roe = index_con_roe[(index_con_roe['TRADE_DATE'] >= self.start_date) & (index_con_roe['TRADE_DATE'] <= self.end_date)]
        index_con_roe_yoy = pd.read_excel('{0}index_con_roe_yoy.xlsx'.format(data_path))
        index_con_roe_yoy = index_con_roe_yoy.set_index('DateTime').unstack().reset_index()
        index_con_roe_yoy.columns = ['INDEX_SYMBOL', 'TRADE_DATE', 'CON_ROE_YOY']
        index_con_roe_yoy['TRADE_DATE'] = index_con_roe_yoy['TRADE_DATE'].apply(lambda x: str(x).replace('-', '')[:8])
        index_con_roe_yoy = index_con_roe_yoy[index_con_roe_yoy['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        index_con_roe_yoy = index_con_roe_yoy[(index_con_roe_yoy['TRADE_DATE'] >= self.start_date) & (index_con_roe_yoy['TRADE_DATE'] <= self.end_date)]
        index_pb_lf = pd.read_excel('{0}index_pb_lf.xlsx'.format(data_path))
        index_pb_lf = index_pb_lf.set_index('DateTime').unstack().reset_index()
        index_pb_lf.columns = ['INDEX_SYMBOL', 'TRADE_DATE', 'PB_LF']
        index_pb_lf['TRADE_DATE'] = index_pb_lf['TRADE_DATE'].apply(lambda x: str(x).replace('-', '')[:8])
        index_pb_lf = index_pb_lf[index_pb_lf['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        index_pb_lf = index_pb_lf[(index_pb_lf['TRADE_DATE'] >= self.start_date) & (index_pb_lf['TRADE_DATE'] <= self.end_date)]
        index_pe_ttm = self.index_valuation.copy(deep=True)
        index_pe_ttm['TRADE_DATE'] = index_pe_ttm['TRADE_DATE'].apply(lambda x: str(x).replace('-', ''))
        index_pe_ttm = index_pe_ttm[index_pe_ttm['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        index_pe_ttm = index_pe_ttm[(index_pe_ttm['TRADE_DATE'] >= self.start_date) & (index_pe_ttm['TRADE_DATE'] <= self.end_date)]

        for i, index in enumerate(self.indexs[self.indexs['板块'] == '先进制造']['代码'].unique().tolist()):
            index_pb_lf_index = index_pb_lf[index_pb_lf['INDEX_SYMBOL'] == index]
            index_finance_index = index_finance[index_finance['INDEX_SYMBOL'] == index]
            max_report_date = index_finance_index.dropna(subset=['MAIN_INCOME'])['REPORT_DATE'].max()
            index_finance_index = index_finance_index[index_finance_index['REPORT_DATE'] <= max_report_date]
            index_finance_index = index_finance_index.merge(self.calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
            index_pb_lf_index = index_pb_lf_index[['TRADE_DATE', 'INDEX_SYMBOL', 'PB_LF']]
            index_finance_index = index_finance_index[['TRADE_DATE', 'INDEX_SYMBOL', 'ROE']]
            index_pb_roe = index_finance_index.merge(index_pb_lf_index, on=['TRADE_DATE', 'INDEX_SYMBOL'], how='left')
            index_pb_roe = index_pb_roe.fillna(method='ffill')
            index_pb_roe['PB_LF'] = np.log(index_pb_roe['PB_LF'])
            index_pb_roe = index_pb_roe.sort_values('TRADE_DATE', ascending=False).reset_index().drop('index', axis=1)
            plt.figure(figsize=(6, 6))
            sns.lmplot(x='ROE', y='PB_LF', data=index_pb_roe, scatter=False, line_kws={'color': 'm'})
            plt.plot(index_pb_roe['ROE'].values, index_pb_roe['PB_LF'].values, linestyle='dotted', color=line_color_list[2])
            norm = matplotlib.colors.Normalize(vmin=0, vmax=len(index_pb_roe))
            for i in range(len(index_pb_roe)):
                c_rgb = matplotlib.cm.Purples_r(norm(i), bytes=True)
                c = from_rgb_to_color16(list(c_rgb)[:3])
                plt.plot(index_pb_roe['ROE'].iloc[i], index_pb_roe['PB_LF'].iloc[i], '*', color=c)
            plt.gca().xaxis.set_major_formatter(FuncFormatter(to_percent))
            plt.xlabel('预期ROE（历史ROE）', fontsize=10)
            plt.ylabel('ln(PB)', fontsize=10)
            plt.title(indexs_names_dic[index], fontsize=10)
            plt.tight_layout()
            plt.savefig('{0}pb_roe/roe_{1}_{2}'.format(self.data_path, indexs_names_dic[index], self.end_date))

        max_report_date = index_finance.pivot(index='REPORT_DATE', columns='INDEX_SYMBOL', values='MAIN_INCOME').dropna().index.max()
        index_finance_latest = index_finance[index_finance['REPORT_DATE'] == max_report_date]
        index_finance_latest = index_finance_latest[index_finance_latest['INDEX_SYMBOL'].isin(self.indexs[self.indexs['板块'] == '先进制造']['代码'].unique().tolist())]
        index_finance_latest = index_finance_latest.merge(self.calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
        index_finance_latest = index_finance_latest[['TRADE_DATE', 'INDEX_SYMBOL', 'ROE']]
        index_pb_lf_latest = index_pb_lf[['TRADE_DATE', 'INDEX_SYMBOL', 'PB_LF']]
        index_pb_roe = index_finance_latest.merge(index_pb_lf_latest, on=['TRADE_DATE', 'INDEX_SYMBOL'], how='left')
        index_pb_roe['PB_LF'] = np.log(index_pb_roe['PB_LF'])
        plt.figure(figsize=(6, 6))
        sns.lmplot(x='ROE', y='PB_LF', data=index_pb_roe, scatter=True, scatter_kws={'color': 'm'}, line_kws={'color': 'm'})
        for i in range(len(index_pb_roe)):
            plt.annotate(indexs_names_dic[index_pb_roe['INDEX_SYMBOL'][i]], xy=(index_pb_roe['ROE'][i], index_pb_roe['PB_LF'][i]), xytext=(index_pb_roe['ROE'][i], index_pb_roe['PB_LF'][i]))  # 这里xy是需要标记的坐标，xytext是对应的标签坐标
        plt.xlabel('预期ROE（历史ROE）')
        plt.ylabel('ln(PB)')
        plt.gca().xaxis.set_major_formatter(FuncFormatter(to_percent))
        plt.tight_layout()
        plt.savefig('{0}pb_roe/roe_{1}'.format(self.data_path, self.end_date))

        for i, index in enumerate(self.indexs[self.indexs['板块'] == '先进制造']['代码'].unique().tolist()):
            index_pb_lf_index = index_pb_lf[index_pb_lf['INDEX_SYMBOL'] == index]
            index_finance_index = index_finance[index_finance['INDEX_SYMBOL'] == index]
            max_report_date = index_finance_index.dropna(subset=['MAIN_INCOME'])['REPORT_DATE'].max()
            index_finance_index = index_finance_index[index_finance_index['REPORT_DATE'] <= max_report_date]
            index_finance_index = index_finance_index.merge(self.calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
            index_pb_lf_index = index_pb_lf_index[['TRADE_DATE', 'INDEX_SYMBOL', 'PB_LF']]
            index_finance_index = index_finance_index[['TRADE_DATE', 'INDEX_SYMBOL', 'ROE_Q']]
            index_pb_roe = index_finance_index.merge(index_pb_lf_index, on=['TRADE_DATE', 'INDEX_SYMBOL'], how='left')
            index_pb_roe = index_pb_roe.fillna(method='ffill')
            index_pb_roe['PB_LF'] = np.log(index_pb_roe['PB_LF'])
            index_pb_roe = index_pb_roe.sort_values('TRADE_DATE', ascending=False).reset_index().drop('index', axis=1)
            plt.figure(figsize=(6, 6))
            sns.lmplot(x='ROE_Q', y='PB_LF', data=index_pb_roe, scatter=False, line_kws={'color': 'm'})
            plt.plot(index_pb_roe['ROE_Q'].values, index_pb_roe['PB_LF'].values, linestyle='dotted', color=line_color_list[2])
            norm = matplotlib.colors.Normalize(vmin=0, vmax=len(index_pb_roe))
            for i in range(len(index_pb_roe)):
                c_rgb = matplotlib.cm.Purples_r(norm(i), bytes=True)
                c = from_rgb_to_color16(list(c_rgb)[:3])
                plt.plot(index_pb_roe['ROE_Q'].iloc[i], index_pb_roe['PB_LF'].iloc[i], '*', color=c)
            plt.gca().xaxis.set_major_formatter(FuncFormatter(to_percent))
            plt.xlabel('预期ROE（单季度ROE）', fontsize=10)
            plt.ylabel('ln(PB)', fontsize=10)
            plt.title(indexs_names_dic[index], fontsize=10)
            plt.tight_layout()
            plt.savefig('{0}pb_roe/roe_q_{1}_{2}'.format(self.data_path, indexs_names_dic[index], self.end_date))

        max_report_date = index_finance.pivot(index='REPORT_DATE', columns='INDEX_SYMBOL', values='MAIN_INCOME').dropna().index.max()
        index_finance_latest = index_finance[index_finance['REPORT_DATE'] == max_report_date]
        index_finance_latest = index_finance_latest[index_finance_latest['INDEX_SYMBOL'].isin(self.indexs[self.indexs['板块'] == '先进制造']['代码'].unique().tolist())]
        index_finance_latest = index_finance_latest.merge(self.calendar_trade_df.rename(columns={'CALENDAR_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
        index_finance_latest = index_finance_latest[['TRADE_DATE', 'INDEX_SYMBOL', 'ROE_Q']]
        index_pb_lf_latest = index_pb_lf[['TRADE_DATE', 'INDEX_SYMBOL', 'PB_LF']]
        index_pb_roe = index_finance_latest.merge(index_pb_lf_latest, on=['TRADE_DATE', 'INDEX_SYMBOL'], how='left')
        index_pb_roe['PB_LF'] = np.log(index_pb_roe['PB_LF'])
        plt.figure(figsize=(6, 6))
        sns.lmplot(x='ROE_Q', y='PB_LF', data=index_pb_roe, scatter=True, scatter_kws={'color': 'm'}, line_kws={'color': 'm'})
        for i in range(len(index_pb_roe)):
            plt.annotate(indexs_names_dic[index_pb_roe['INDEX_SYMBOL'][i]], xy=(index_pb_roe['ROE_Q'][i], index_pb_roe['PB_LF'][i]), xytext=(index_pb_roe['ROE_Q'][i], index_pb_roe['PB_LF'][i]))  # 这里xy是需要标记的坐标，xytext是对应的标签坐标
        plt.xlabel('预期ROE（单季度ROE）')
        plt.ylabel('ln(PB)')
        plt.gca().xaxis.set_major_formatter(FuncFormatter(to_percent))
        plt.tight_layout()
        plt.savefig('{0}pb_roe/roe_q_{1}'.format(self.data_path, self.end_date))

        for i, index in enumerate(self.indexs[self.indexs['板块'] == '先进制造']['代码'].unique().tolist()):
            index_pe_ttm_index = index_pe_ttm[index_pe_ttm['INDEX_SYMBOL'] == index]
            index_con_roe_index = index_con_roe[index_con_roe['INDEX_SYMBOL'] == index]
            index_pe_ttm_index = index_pe_ttm_index[['TRADE_DATE', 'INDEX_SYMBOL', 'PE_TTM']]
            index_con_roe_index = index_con_roe_index[['TRADE_DATE', 'INDEX_SYMBOL', 'CON_ROE_FY1']]
            index_pb_roe = index_pe_ttm_index.merge(index_con_roe_index, on=['TRADE_DATE', 'INDEX_SYMBOL'], how='left')
            index_pb_roe = index_pb_roe.fillna(method='ffill')
            index_pb_roe = index_pb_roe.merge(self.trade_df, on=['TRADE_DATE'], how='left')
            index_pb_roe = index_pb_roe.sort_values('TRADE_DATE', ascending=True).reset_index().drop('index', axis=1)
            index_pb_roe_weekly = index_pb_roe[index_pb_roe['IS_WEEK_END'] == '1']
            index_pb_roe_weekly = index_pb_roe_weekly.sort_values('TRADE_DATE', ascending=False).reset_index().drop('index', axis=1)
            plt.figure(figsize=(6, 6))
            # sns.lmplot(x='CON_ROE_FY1', y='PE_TTM', data=index_pb_roe, scatter=False, line_kws={'color': 'm'})
            # plt.plot(index_pb_roe['CON_ROE_FY1'].values, index_pb_roe['PB_LF'].values, linestyle='dotted', color=line_color_list[2])
            norm = matplotlib.colors.Normalize(vmin=0, vmax=len(index_pb_roe_weekly))
            for i in range(len(index_pb_roe_weekly)):
                c_rgb = matplotlib.cm.Purples_r(norm(i), bytes=True)
                c = from_rgb_to_color16(list(c_rgb)[:3])
                plt.plot(index_pb_roe_weekly['CON_ROE_FY1'].iloc[i], index_pb_roe_weekly['PE_TTM'].iloc[i], '*', color=c)
            plt.gca().xaxis.set_major_formatter(FuncFormatter(to_percent))
            plt.xlabel('预期ROE（一致预测ROE）', fontsize=10)
            plt.ylabel('PE_TTM', fontsize=10)
            plt.title(indexs_names_dic[index], fontsize=10)
            plt.tight_layout()
            plt.savefig('{0}pb_roe/con_roe_{1}_{2}'.format(self.data_path, indexs_names_dic[index], self.end_date))

        index_con_roe_latest = index_con_roe[index_con_roe['TRADE_DATE'] == self.end_date]
        index_con_roe_latest = index_con_roe_latest[index_con_roe_latest['INDEX_SYMBOL'].isin(self.indexs[self.indexs['板块'] == '先进制造']['代码'].unique().tolist())]
        index_con_roe_latest = index_con_roe_latest[['TRADE_DATE', 'INDEX_SYMBOL', 'CON_ROE_FY1']]
        index_pe_ttm_latest = index_pe_ttm[index_pe_ttm['TRADE_DATE'] == self.end_date]
        index_pe_ttm_latest = index_pe_ttm_latest[index_pe_ttm_latest['INDEX_SYMBOL'].isin(self.indexs[self.indexs['板块'] == '先进制造']['代码'].unique().tolist())]
        index_pe_ttm_latest = index_pe_ttm_latest[['TRADE_DATE', 'INDEX_SYMBOL', 'PE_TTM']]
        index_pb_roe = index_con_roe_latest.merge(index_pe_ttm_latest, on=['TRADE_DATE', 'INDEX_SYMBOL'], how='left')
        plt.figure(figsize=(6, 6))
        plt.scatter(index_pb_roe['CON_ROE_FY1'].values, index_pb_roe['PE_TTM'].values, color='m')
        for i in range(len(index_pb_roe)):
            plt.annotate(indexs_names_dic[index_pb_roe['INDEX_SYMBOL'][i]], xy=(index_pb_roe['CON_ROE_FY1'][i], index_pb_roe['PE_TTM'][i]), xytext=(index_pb_roe['CON_ROE_FY1'][i], index_pb_roe['PE_TTM'][i]))  # 这里xy是需要标记的坐标，xytext是对应的标签坐标
        plt.xlabel('预期ROE（一致预测ROE）')
        plt.ylabel('PE_TTM')
        plt.gca().xaxis.set_major_formatter(FuncFormatter(to_percent))
        plt.tight_layout()
        plt.savefig('{0}pb_roe/con_roe_{1}'.format(self.data_path, self.end_date))
        return

    def get_amt_results(self):
        self.get_amt_ret()
        self.get_amt_valuation()
        self.get_amt_prosperity()
        self.get_amt_con_prosperity()
        self.get_amt_prosperity_factor()
        self.get_amt_pb_roe()


if __name__ == '__main__':
    start_date = '20190101'
    end_date = '20220805'
    data_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/industry_tracking/'
    PeTracking(start_date, end_date, data_path).get_amt_results()
    IndustryTracking(start_date, end_date, data_path).get_amt_results()







"""
股票市场资金类指标
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class StockCashFlow:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_stock_cash_flow'

    def get_stock_cash_flow_data(self):
        """
        股票市场资金类数据：融资融券、北向资金
        """
        # 两融
        index_list = ['M0075992', 'M0075990', 'M0075991']
        name_dict = {"M0075992": "margin", "M0075990": "financing", "M0075991": "sec_lending"}
        res = w.edb(','.join(index_list), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch stock margin data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)
        margin = data.copy()
        # 北向资金
        res = w.wset("shhktransactionstatistics",
                     "startdate={};enddate={};cycle=day;currency=cny;field=date,sh_net_purchases".format(
                         self.start_date, self.end_date))
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch shhk data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            data = pd.DataFrame(res.Data, index=res.Fields).T
            data.rename(columns={"date": "trade_date"}, inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        shhk = data.copy()
        res = w.wset("szhktransactionstatistics",
                     "startdate={};enddate={};cycle=day;currency=cny;field=date,sz_net_purchases".format(
                         self.start_date, self.end_date))
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch szhk data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            data = pd.DataFrame(res.Data, index=res.Fields).T
            data.rename(columns={"date": "trade_date"}, inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        szhk = data.copy()
        hk_cash_df = pd.merge(shhk, szhk, on='trade_date', how='outer')
        cash_flow_df = margin.merge(hk_cash_df, on='trade_date', how='outer').sort_values(by='trade_date')

        return cash_flow_df

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_stock_cash_flow_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                    create table mac_stock_cash_flow(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    margin decimal(10, 2),
                    financing decimal(10, 2),
                    sec_lending decimal(8, 2),
                    sh_net_purchases decimal(6, 2),
                    sz_net_purchases decimal(6, 2)) 
                """
            create_table(self.table_name, sql_script)
            data = self.get_stock_cash_flow_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    StockCashFlow('2010-03-31', '2021-04-23', is_increment=0).get_construct_result()


from enum import Enum
from hbshare.base.upass import is_prod_env

# 办公环境
DOMAINS_OFFICE = {'hbcgi': 'data.howbuy.com', 's': 's.howbuy.com',
                  'ams': 'ams-data.intelnal.howbuy.com', 
                  'ams-admin': 'ams.howbuy.com'
                #   'ams-admin': 'ams.it38.k8s.howbuy.com'
                  }

# 产线环境
DOMAINS_PROD = {'hbcgi': 'data.howbuy.com', 's': 's.howbuy.com',
                'ams': 'ams.inner.howbuy.com', 'ams-admin': 'ams.howbuy.com'}
P_TYPE = {'http': 'http://', 'ftp': 'ftp://', 'https': 'https://'}


class UrlCfg():
    def __init__(self, url, method, domain, parsePostDataFunc, parseParamsFunc, supportFields):
        self.__url__ = url
        self.method = method
        self.__domain__ = domain
        self.__parsePostDataFunc__ = parsePostDataFunc
        self.__parseParamsFunc__ = parseParamsFunc
        self.supportFields = supportFields

    def parsePostData(self, kwargs):
        return self.__parsePostDataFunc__(kwargs)

    def parseParams(self, kwargs):
        return self.__parseParamsFunc__(kwargs)

    def getUrl(self):
        if is_prod_env():
            return self.__url__ % DOMAINS_PROD.get(self.__domain__)
        return self.__url__ % DOMAINS_OFFICE.get(self.__domain__)

    def supportFields(self):
        return self.supportFields


class UrlEnum(Enum):

    @classmethod
    def getValue(self, name):
        '''根据枚举name匹配到枚举对象，并返回对应的value'''
        if name in self._member_names_:
            tmpEnum = self._member_map_[name]
            return tmpEnum.value
        else:
            raise ValueError("%s is not a valid UrlEnum" % name)

############################### 指数 ###############################
    # 查询指数行情
    MARKET_HQ = UrlCfg('http://%s/data/fund/brinson/zsjy', 'post', 'ams',
                       lambda x: {
                           'zqdm': x['zqdm'],
                           'jyrq': {
                               'startDate': x['startDate'],
                               'endDate': x['endDate']
                           },
                           'fields': x['fields'],
                           'page': x['page'],
                           'perPage': x['perPage']
                       },
                       lambda y: {},
                       supportFields=('jlzj', 'jyrq', 'scdm', 'zqdm', 'zqmc', 'qspj', 'kpjg', 'spjg', 'zgjg', 'zdjg', 'cjsl', 'cjjs',
                                      'cjbs', 'zdsl', 'zdfd', 'bdfd', 'hbjn', 'hb1z', 'hb4z', 'hb13z', 'hb26z', 'hb52z', 'hb1y', 'hb3y', 'hb6y',
                                      'hb1n', 'ggrq', 'recstat', 'checkflag', 'creator', 'modifier', 'checker', 'credt', 'moddt', 'stimestamp',
                                      'hb2n', 'hb3n', 'hb5n', 'hbdr', 'fl', 'pb', 'roe', 'gxl', 'pebfw', 'pbbfw', 'syl', 'nhsy1y', 'nhsy1n', 'nhsy2n',
                                      'nhsy3n', 'nhsy3y', 'nhsy6y', 'nhsy5n', 'gxlbfw', 'ltsz', 'zsz',)
                       )
    ''' 查询指数行情 '''

################################ 公募 ################################
    # 公募基金回报
    FUND_JJHB = UrlCfg('http://%s/data/fund/brinson/jjhb', 'post', 'ams',
                       lambda x: {
                           'jjdm': x['jjdm'],
                           'jzrq': {
                               'startDate': x['startDate'],
                               'endDate': x['endDate']
                           },
                           'fields': x['fields'],
                           'page': x['page'],
                           'perPage': x['perPage']
                       },
                       lambda y: {},
                       supportFields=('jzrq', 'jjdm', 'hbdr', 'hb1z', 'hb1y', 'hb2y', 'hb3y', 'hb4y', 'hb6y', 'hbjn', 'hb1n', 'hb2n',
                                      'hb3n', 'hb4n', 'hb5n', 'hbcl', 'hbfh', 'pm30r', 'pm60r', 'pm90r', 'pm180r', 'pmdr', 'pm1z', 'pm1y',
                                      'pm3y', 'pm6y', 'pmjn', 'pm1n', 'pm2n', 'pm3n', 'pm4n', 'pm5n', 'pmcl', 'qrsy', 'pm7r', 'wjsy',
                                      'recstat', 'checkflag', 'creator', 'modifier', 'checker', 'credt', 'moddt', 'stimestamp', 'hb2z',
                                      'hb3z', 'pm2z', 'pm3z', 'pmdr2', 'pm1z2', 'pm1y2', 'pm3y2', 'pm6y2', 'pmjn2', 'pm1n2', 'pm2n2', 'pm3n2',
                                      'pm4n2', 'pm5n2', 'pmcl2', 'pm2z2', 'pm3z2', 'nhsyl1y', 'nhsy3y', 'nhsy6y', 'nhsy12y', 'jbdl1y',
                                      'pmbzf1y', 'pmbzf3y', 'pmbzf1n', 'hmtjzs', 'sswx', 'xswx', 'nhsy14r', 'nhsy28r', 'nhsy30r', 'nhsy35r',
                                      'nhsy60r', 'nhsy90r', 'nhsy180r', 'nhsy1y', 'nhsy1n', 'nhsy2n', 'nhsy3n', 'nhsy5n', 'bfbpm3y', 'bfbpm6y',
                                      'bfbpm1n', 'bfbpm2n', 'bfbpm3n', 'bfbpm5n', 'bfbpmjn', 'bfbpm3y2', 'bfbpm6y2', 'bfbpm1n2', 'bfbpm3n2', 'zddr',)
                       )
    '''公募基金回报'''
    # 公募基金公募持仓数据
    FUND_JJ_GPZH = UrlCfg('http://%s/data/fund/brinson/gpzh', 'post', 'ams',
                          lambda x: {
                              'jjdm': x['jjdm'],
                              'ggrq': {
                                  'startDate': x['startDate'],
                                  'endDate': x['endDate']
                              },
                              'fields': x['fields'],
                              'page': x['page'],
                              'perPage': x['perPage']
                          },
                          lambda y: {},
                          supportFields=('jjdm', 'qsrq', 'jsrq', 'zqdm', 'zqmc', 'ltms', 'tzlx', 'sszt', 'ccsz', 'ccsl',
                                         'zjbl', 'zgbl', 'zlbl', 'jzgj', 'jlfz', 'ggrq', 'recstat', 'checkflag',
                                         'creator', 'modifier', 'checker', 'credt', 'moddt', 'stimestamp', 'sftj',
                                         'szbdbl', 'slbdbl', 'zjblbdbl', 'zgblbd', 'zgblbdbl', 'zlblbdbl',)
                          )
    '''公募基金公募持仓数据'''
    # 公募持仓资产比例数据
    FUND_JJ_ZCPZ = UrlCfg('http://%s/data/fund/brinson/zcpz', 'post', 'ams',
                          lambda x: {
                              'jjdm': x['jjdm'],
                              'jsrq': {
                                  'startDate': x['startDate'],
                                  'endDate': x['endDate']
                              },
                              'fields': x['fields'],
                              'page': x['page'],
                              'perPage': x['perPage']
                          },
                          lambda y: {},
                          supportFields=('jjdm', 'jsrq', 'hbzl', 'jjzc', 'jzzc', 'zssz', 'jjsz', 'jjbl', 'gpbl',
                                         'qzsz', 'qzbl', 'zzsz', 'cqbl', 'kzbl', 'gzsz', 'gjbl', 'jzsz', 'jzbl',
                                         'ypsz', 'ypbl', 'qysz', 'qybl', 'qqsz', 'zqsz', 'zqbl', 'yhck', 'ckbl',
                                         'qsbl', 'hbbl', 'gzhb', 'zqhb', 'ysqs', 'yqbl', 'bzbl', 'glbl', 'lxbl',
                                         'sgbl', 'dtbl', 'qtys', 'qttz', 'tzbl', 'dfye', 'fsye', 'fsbl', 'mchg',
                                         'qtfz', 'chbh', 'ggrq', 'recstat', 'checkflag', 'creator', 'modifier',
                                         'checker', 'credt', 'moddt', 'stimestamp', 'gpbd', 'gpjz', 'gjbd', 'hbbd',
                                         'jjtzszhj', 'data_source', 'zxqysmzqsz', 'yqtzsz', 'qqtzsz', 'hbscgjszhj',
                                         'zcxjrz', 'qydqrzq', 'zqpj', 'sycxqcg397tzq', 'bz', 'jrysptzbl', 'qhbl',
                                         'ctpzbl', 'zcxjrzbl', 'qydqrzqbl', 'zqpjbl', 'sycxqcg397tzqbl', 'mdshgdfszcbl',
                                         'qt', 'mdshgdfszc', 'yxg', 'fdc', 'gjs', 'zqjz', 'tycd', 'dfzfz', 'qytz', 'gdsytz', 'cqgqtz',)
                          )
    '''公募持仓资产比例数据'''

    # 指数成分股数据
    MARKET_CFG = UrlCfg('http://%s/data/fund/brinson/lc_icw', 'post', 'ams',
                        lambda x: {
                            'secuCode': x['secuCode'],
                            'endDate': x['endDate'],
                            'fields': x['fields'],
                            'page': x['page'],
                            'perPage': x['perPage']
                        },
                        lambda y: {},
                        supportFields=('indexcode', 'innercode', 'infosource',
                                       'enddate', 'weight', 'updatetime', 'jsid', 'secucode',)
                        )
    '''指数成分股数据'''

    # 公私募净值归因 - Barra风格因子数据
    BARRA_FACTORY = UrlCfg('http://%s/data/fund/jzgy/barra_factor_return', 'post', 'ams',
                           lambda x: {
                               'tradeDate': {
                                   'startDate': x['startDate'],
                                   'endDate': x['endDate']
                               },
                               'fields': x['fields'],
                               'page': x['page'],
                               'perPage': x['perPage']
                           },
                           lambda y: {},
                           supportFields=(
                               'tradeDate', 'factorName', 'factorRet',)
                           )
    '''公私募净值归因 - Barra风格因子数据'''

    #  全量指数行情数据
    MARKET_HQQL = UrlCfg('http://%s/data/fund/jzgy/hqql', 'post', 'ams',
                         lambda x: {
                             'zqdm': x['zqdm'],
                             'jyrq': {
                                 'startDate': x['startDate'],
                                 'endDate': x['endDate']
                             },
                             'fields': x['fields'],
                             'page': x['page'],
                             'perPage': x['perPage']
                         },
                         lambda y: {},
                         supportFields=('zqdm', 'scdm', 'jyrq', 'zqmc', 'qspj', 'kpjg', 'spjg',
                                        'zgjg', 'zdjg', 'cjsl', 'cjjs', 'cjbs', 'zdsl', 'zdfd',
                                        'bdfd', 'ltsz', 'zsz', 'pb', 'pe', 'roe', 'gxl',)
                         )
    ''' 全量指数行情数据'''

    #  公私募净值归因 - 板块指数数据
    SECTOR_FACTOR = UrlCfg('http://%s/data/fund/jzgy/sector_factor', 'post', 'ams',
                           lambda x: {
                               'tradeDate': {
                                   'startDate': x['startDate'],
                                   'endDate': x['endDate']
                               },
                               'fields': x['fields'],
                               'page': x['page'],
                               'perPage': x['perPage']
                           },
                           lambda y: {},
                           supportFields=(
                               'tradeDate', 'bigfinance', 'consuming', 'tmt', 'cycle', 'manufacture',)
                           )
    ''' 公私募净值归因 - 板块指数数据 '''

    #  私募基金回报数据
    SIMU_RHB = UrlCfg('http://%s/data/fund/jzgy/rhb', 'post', 'ams',
                      lambda x: {
                          'jjdm': x['jjdm'],
                          'jzrq': {
                              'startDate': x['startDate'],
                              'endDate': x['endDate']
                          },
                          'fields': x['fields'],
                          'page': x['page'],
                          'perPage': x['perPage']
                      },
                      lambda y: {},
                      supportFields=('jjdm', 'jzrq', 'hbdr',
                                     'hbcl', 'hbfh', 'fqdwjz',)
                      )
    ''' 私募基金回报数据 '''

################################ A股市场/科创板 ################################
    # A股市场估值数据
    SC_VALUATION_A = UrlCfg('http://%s/data/fund/sac/dfv_stibdifv_a', 'post', 'ams',
                            lambda x: {
                                'tradingDay': {
                                    'startDate': x['startDate'],
                                    'endDate': x['endDate']
                                },
                                'fields': x['fields'],
                                'page': x['page'],
                                'perPage': x['perPage']
                            },
                            lambda y: {},
                            supportFields=('innercode', 'tradingday', 'pe', 'pb', 'pcf', 'ps', 'dividendratio', 'updatetime', 'jsid',
                                           'psttm', 'totalmv', 'negotiablemv', 'pelyr', 'pcfttm', 'pcfs', 'pcfsttm', 'dividendratiolyr',
                                           'enterprisevaluew', 'enterprisevaluen', 'inserttime', 'totalmv2', 'totalasmv', 'pettmcut',
                                           'forwardpeqa', 'forwardpehr', 'peg', 'pettmtoorrate', 'forwardpcfqa', 'forwardpcfhr',
                                           'forwardpcfsqa', 'forwardpcfshr', 'forwardpsqa', 'forwardpshr', 'dividendratiolyr2',
                                           'dividendratio2', 'evtoebitda', 'evtoor', 'evtofcff', 'evtocfo', 'evtogp', 'per',
                                           'evtoebitdarandd', 'totalmvtorandd', 'mva', 'floatmva', 'marketvaluetodebt1',
                                           'marketvaluetodebt2', 'debttoassetvalue1', 'debttoassetvalue2',)
                            )
    ''' A股市场估值数据 '''
    # 科创板估值数据
    SC_VALUATION_KC = UrlCfg('http://%s/data/fund/sac/dfv_stibdifv_kc', 'post', 'ams',
                             lambda x: {
                                 'tradingDay': {
                                     'startDate': x['startDate'],
                                     'endDate': x['endDate']
                                 },
                                 'fields': x['fields'],
                                 'page': x['page'],
                                 'perPage': x['perPage']
                             },
                             lambda y: {},
                             supportFields=('innercode', 'tradingday', 'pe', 'pb', 'pcf', 'ps', 'dividendratio', 'updatetime', 'jsid',
                                            'psttm', 'totalmv', 'negotiablemv', 'pelyr', 'pcfttm', 'pcfs', 'pcfsttm', 'dividendratiolyr',
                                            'enterprisevaluew', 'enterprisevaluen', 'inserttime', 'totalmv2', 'totalasmv', 'pettmcut',
                                            'forwardpeqa', 'forwardpehr', 'peg', 'pettmtoorrate', 'forwardpcfqa', 'forwardpcfhr',
                                            'forwardpcfsqa', 'forwardpcfshr', 'forwardpsqa', 'forwardpshr', 'dividendratiolyr2',
                                            'dividendratio2', 'evtoebitda', 'evtoor', 'evtofcff', 'evtocfo', 'evtogp', 'per',
                                            'evtoebitdarandd', 'totalmvtorandd', 'mva', 'floatmva', 'marketvaluetodebt1',
                                            'marketvaluetodebt2', 'debttoassetvalue1', 'debttoassetvalue2',)
                             )
    ''' 科创板估值数据 '''
    # A股市场复权行情数据
    SC_FQ_A = UrlCfg('http://%s/data/fund/brinson/pd_stibpd_a', 'post', 'ams',
                     lambda x: {
                         'tradingDay': {
                             'startDate': x['startDate'],
                             'endDate': x['endDate']
                         },
                         'fields': x['fields'],
                         'page': x['page'],
                         'perPage': x['perPage']
                     },
                     lambda y: {},
                     supportFields=('id', 'innercode', 'tradingday', 'closeprice', 'changepct', 'backwardprice',
                                    'risingupdays', 'fallingdowndays', 'maxrisingupdays', 'maxfallingdowndays',
                                    'fallondebut', 'fallonnaps', 'ahpremiumrate50', 'stockboard', 'limitboard',
                                    'surgedlimit', 'declinelimit', 'highestprice', 'lowestprice', 'highestpricerw',
                                    'lowestpricerw', 'highestpricetw', 'lowestpricetw', 'highestpricerm', 'lowestpricerm',
                                    'highestpricetm', 'lowestpricetm', 'highestpricer3m', 'lowestpricer3m', 'highestpricer6m',
                                    'lowestpricer6m', 'highestpriceytd', 'lowestpriceytd', 'highestpricer12m', 'lowestpricer12m',
                                    'updatetime', 'jsid', 'highestpricermthree', 'lowestpricermthree', 'highestpricermsix',
                                    'lowestpricermsix', 'highestpricery', 'lowestpricery', 'inserttime',)
                     )
    '''A股市场复权行情数据'''
    # 科创板复权行情数据
    SC_FQ_KC = UrlCfg('http://%s/data/fund/brinson/pd_stibpd_kc', 'post', 'ams',
                      lambda x: {
                          'tradingDay': {
                              'startDate': x['startDate'],
                              'endDate': x['endDate']
                          },
                          'fields': x['fields'],
                          'page': x['page'],
                          'perPage': x['perPage']
                      },
                      lambda y: {},
                      supportFields=('id', 'innercode', 'tradingday', 'closeprice', 'changepct', 'backwardprice',
                                     'risingupdays', 'fallingdowndays', 'maxrisingupdays', 'maxfallingdowndays',
                                     'fallondebut', 'fallonnaps', 'ahpremiumrate50', 'stockboard', 'limitboard',
                                     'surgedlimit', 'declinelimit', 'highestprice', 'lowestprice', 'highestpricerw',
                                     'lowestpricerw', 'highestpricetw', 'lowestpricetw', 'highestpricerm', 'lowestpricerm',
                                     'highestpricetm', 'lowestpricetm', 'highestpricer3m', 'lowestpricer3m', 'highestpricer6m',
                                     'lowestpricer6m', 'highestpriceytd', 'lowestpriceytd', 'highestpricer12m', 'lowestpricer12m',
                                     'updatetime', 'jsid', 'highestpricermthree', 'lowestpricermthree', 'highestpricermsix',
                                     'lowestpricermsix', 'highestpricery', 'lowestpricery', 'inserttime',)
                      )
    '''科创板复权行情数据'''
    # 非标数据日净值 - 日净值分页查询
    FB_RJZ = UrlCfg('http://%s/admin/dailyNetValue/queryList', 'post', 'ams-admin',
                    lambda x: {
                        'belongDate': {
                            'startDate': x.get('startDate'),
                            'endDate': x.get('endDate')
                        },
                        'jjdm': x.get('jjdm'),
                        # 接口查询 人员下的基金代码 再拼接 参数基金代码去做查询。
                        'rydm': x.get('rydm'),
                        'sorts': [{'jjdm': 'desc', 'jzrq': 'asc'}],
                        'page': x['page'],
                        'perPage': x['perPage']
                    },
                    lambda y: {'access_token': y['access_token']},
                    supportFields=('jjdm', 'jjmc', 'netValue', 'netValueDate', 'fqdwjz', 'ljjz',
                                   'glry', 'glrymc', 'researchers', 'updateTime',
                                   'updateUsername', 'createdUser', 'createdUsername', 'createdTime')
                    )
    '''非标数据日净值 - 日净值分页查询'''
    # fof估值子基金 - 查询子基金列表
    FOF_ZJJLIST = UrlCfg('http://%s/admin/fof/gz/sf/list', 'post', 'ams-admin',
                         lambda x: {
                             'jzrq': {
                                 'startDate': x.get('startDate'),
                                 'endDate': x.get('endDate')
                             },
                             'jjdm': x.get('jjdm'),
                             'khdm': x.get('khdm'),
                             #   数据状态 1-正常 0-异常 不传为 null
                             'state': x.get('state'),
                             'sorts': [{'jjdm': 'desc', 'jzrq': 'asc'}],
                             'page': x['page'],
                             'perPage': x['perPage']
                         },
                         lambda y: {'access_token': y['access_token']},
                         supportFields=('dataSource', 'sjly', 'jjdm', 'originalJjdm', 'jjmc', 'khdm', 'originalKhdm',
                                        'khmc', 'jzrq', 'jjjz', 'ljjz', 'xnjz', 'khzcfe', 'khzcjz', 'jsrq', 'ferq',
                                        'febdrq', 'qrrq', 'fsfe', 'xnyjbc', 'yjbclx', 'jtfs', 'jtqzcjz', 'jtqzcjz2',
                                        'scjtrq', 'mrdwjz', 'mrljdwjz', 'scjtdwjz', 'scjtljdwjz', 'glr', 'tgr', 'jjzh',
                                        'jyzh', 'khlx', 'zjlx', 'zjhm', 'xsjg', 'ztfhtx', 'ztshtx', 'bz', 'state')
                         )
    '''fof估值子基金 - 查询子基金列表'''



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# @FileName  :BaseIndex.py
# @Time      :2021/4/25 4:49 下午
# @Author    :meng.lv
import hbshare as hbs
import pandas as pd
import numpy as np
import sys, os
from datetime import datetime, timedelta

class BaseIndex(object):

    def __init__(self, database, path):
        self.TIME_DICT = {"DAY": "jzrq", "WEEK": "tjrq", "MONTH": "tjyf", "YEAR": "tjnf"}
        self.database = database
        hbs.set_token("qwertyuisdfghjkxcvbn1000")
        self.config = self.init_config(path)

    def get_config(self):
        order = ['NAME_CN', 'KEY', 'DOMAIN', 'DAY', 'WEEK', 'MONTH', 'YEAR']
        return self.config[order]

    def call(self, sql):
        data = hbs.db_data_query(self.database, sql)
        if data['returnCode'] != 'F0000000':
            raise QueryException(data)
        return pd.DataFrame.from_dict(data['data'])

    def sql(self, k):
        key, dim_time = self.split_key(k)
        df = self.config
        if dim_time not in list(self.TIME_DICT.keys()):
            raise KeyError(k, " 请检查时间维定义是否正确 ")

        if key not in df.index:
            raise KeyError(key, " 该指标不存在 ")

        if dim_time not in df.columns:
            raise KeyError(dim_time, " 配置中不存在该时间维度 ")

        if df.loc[key][dim_time] == "":
            raise KeyError(k, " 请和DB团队确认该指标是否计算!")

        # print(str(df.loc[key]['NAME_CN']))
        return "select * from st_hedge." + str(df.loc[key][dim_time]) \
               + " a where a.jjdm ='%s' and " + self.TIME_DICT[dim_time] + "='%s'"

    def split_key(self, k):
        #first_idx = k.lfind("_")
        last_idx = k.rfind("_")
        #domain = k[0:first_idx]
        key = k[0:last_idx]
        dim_time = k[last_idx + 1:len(k)]
        return key, dim_time

    def check_dim_time(self, k, dim_time):
        key, dt = self.split_key(k)

        # df = self.get_all_config()
        # print(df.loc["VOLATILITY"]['MONTH'])
        # print(df.index.isin(['VOLATILITY_RANK']))
        if dt not in list(self.TIME_DICT.keys()):
            raise KeyError(k, " 请设置正确的时间维度格式 ")
        if dt == 'DAY' and len(dim_time) != 8:
            raise KeyError(dim_time, " 日频指标请确定时间维度格式：yyyymmdd ")
        if dt == 'WEEK' and len(dim_time) != 8:
            raise KeyError(dim_time, " 周频指标请确定时间维度格式：yyyymmdd ")
        if dt == 'MONTH' and len(dim_time) != 6:
            raise KeyError(dim_time, " 月频指标请确定时间维度格式：yyyymm ")
        if dt == 'YEAR' and len(dim_time) != 4:
            raise KeyError(dim_time, " 年度指标请确定时间维度格式：yyyy ")

        if dt == 'WEEK' and len(dim_time) == 8:
            tjrq = datetime.strptime(str(dim_time), "%Y%m%d")
            dim_time = datetime.strftime(tjrq - timedelta(tjrq.weekday()), "%Y%m%d")
        return dim_time


    def init_config(self, config_path):
        df = pd.read_csv(config_path)
        df['HBS_KEY'] = df['DOMAIN'] + "_" + df['KEY']
        df = df.set_index('HBS_KEY')
        # df = df.fillna(None)
        df = df.replace(np.nan, '', regex=True)
        return df



import numpy as np
import re
import pdfplumber
import pandas as pd
import os
from datetime import datetime
from hbshare.quant.CChen.sql_cons import gtja_gzb_col_sql
from hbshare.quant.CChen.func import generate_table


def str_2_num(x):
    return np.nan if x in ['', ' '] else float(x.replace(',', '')) if type(x) == str else x


def gzb_str_2_num(df):
    df[['数量', '单位成本', '市价', '估值增值', '成本', '市值', '成本占净值', '市值占净值']] = df[
        ['数量', '单位成本', '市价', '估值增值', '成本', '市值', '成本占净值', '市值占净值']
    ].applymap(str_2_num)
    return df


def gen_gzb_table(table, db, sql_info):
    generate_table(
        database=db,
        table=table,
        generate_sql=gtja_gzb_col_sql,
        sql_ip=sql_info['ip'],
        sql_user=sql_info['user'],
        sql_pass=sql_info['pass'],
        table_comment='基金估值表'
    )
    print(table + ' generated')


def ht_transfer(x):
    # 华泰估值表科目名称去除冗余字段
    if type(x) == str:
        r = re.findall(r'[A-Za-z]+', x)
        if len(r):
            return re.findall(r'[A-Za-z0-9]+', x)[0]
        else:
            return x
    else:
        return x


# 用于读取华泰估值表
def ht_gzb(file_path):
    data = pd.read_excel(file_path)
    name = data.iloc[1, 0].replace('华泰证券股份有限公司_', '').replace('_专用表', '')
    name = name.replace('私募证券投资基金', '').replace('私募投资基金', '').replace('私募基金', '')
    date = datetime.strptime(data.iloc[2, 0][-10:], '%Y-%m-%d').date()
    data = data.iloc[:, list(range(6)) + [7, 8, 9, 11, 12, 14, 15]]
    data.columns = data.iloc[3]
    data = data.iloc[6:].reset_index(drop=True)
    df = data.rename(
        columns={
            '成本占比': '成本占净值',
            '行情': '市价',
            '市值占比': '市值占净值',
        }
    ).drop(columns=['币种', '汇率'])
    df_col = df.columns.tolist()
    df['基金名称'] = name
    df['日期'] = date
    df['成本占净值'] = df['成本占净值'] * 100
    df['市值占净值'] = df['市值占净值'] * 100
    df = gzb_str_2_num(df)
    df['科目名称'] = df['科目名称'].apply(ht_transfer)
    return df[['日期', '基金名称'] + df_col]


# 华泰估值表录入数据库
def ht_gzb_to_db(table, fpath, db_path):
    data_exists = pd.read_sql_query('select `日期`, `基金名称` from ' + table, db_path)
    data_exists['label'] = data_exists['基金名称'] + data_exists['日期'].apply(lambda x: x.strftime('%Y%m%d'))
    data_exists = data_exists['label'].tolist()

    file_list = os.listdir(fpath)
    for f in file_list:
        file = fpath + '/' + f
        df = ht_gzb(file_path=file)
        name = df['基金名称'][0]
        date = df['日期'][0]

        if name + date.strftime('%Y%m%d') not in data_exists:
            df.to_sql(table, db_path, index=False, if_exists='append')
            data_exists.append(name + date.strftime('%Y%m%d'))
            print(f)
        else:
            print(name + ' ' + date.strftime('%Y%m%d') + ' exists')


# 用于读取国泰君安估值表
def gtja_gzb(file_path):
    if file_path[-3:].lower() == 'pdf':
        pdf = pdfplumber.open(file_path)

        df = pd.DataFrame()
        name = None
        date = None

        for i in pdf.pages:
            if i.page_number == 1:
                i_text = i.extract_text()
                date = datetime.strptime(i_text[i_text.index('估值日期'):][5:13], '%Y%m%d').date()
                name_index0 = i_text.index('国泰君安证券股份有限公司__')
                name_index1 = i_text.index('__专用表')
                name = i_text[
                         name_index0 + 14: name_index1
                         ].replace('私募证券投资基金', '').replace('私募投资基金', '').replace('私募基金', '')
            table_raw = i.extract_tables()
            table_df = pd.DataFrame(table_raw[0][1:], columns=table_raw[0][0])
            df = pd.concat([df, table_df])
        df.applymap(lambda x: x.replace('\n', ''))
        df['科目名称'] = df['科目名称'].apply(lambda x: x.replace('\n', ''))
        df['科目代码'] = df['科目代码'].apply(lambda x: x.replace('\n', ''))
        df['停牌信息'] = df['停牌信息'].apply(lambda x: x.replace('\n', ''))

    else:
        df = pd.read_excel(file_path)
        name = df.iloc[0, 0].replace('国泰君安证券股份有限公司__', '').replace('国泰君安___', '').replace('___专用表', '')
        name = name.replace('私募证券投资基金', '').replace('私募投资基金', '').replace('私募基金', '')
        date = datetime.strptime(df.iloc[1, 0][-8:], '%Y%m%d').date()
        df.columns = df.iloc[2]
        df = df.iloc[3:].reset_index(drop=True)
    df = df.rename(
        columns={
            '成本占净值%': '成本占净值',
            '市值占净值%': '市值占净值'
        }
    )
    df_col = df.columns.tolist()
    df['基金名称'] = name
    df['日期'] = date
    df = gzb_str_2_num(df)
    return df[['日期', '基金名称'] + df_col].reset_index(drop=True)


# 国泰君安估值表录入数据库
def gtja_gzb_to_db(table, fpath, db_path):
    data_exists = pd.read_sql_query('select `日期`, `基金名称` from ' + table, db_path)
    data_exists['label'] = data_exists['基金名称'] + data_exists['日期'].apply(lambda x: x.strftime('%Y%m%d'))
    data_exists = data_exists['label'].tolist()

    file_list = os.listdir(fpath)
    for f in file_list:
        file = fpath + '/' + f
        df = gtja_gzb(file_path=file)
        name = df['基金名称'][0]
        date = df['日期'][0]

        if name + date.strftime('%Y%m%d') not in data_exists:
            df.to_sql(table, db_path, index=False, if_exists='append')
            data_exists.append(name + date.strftime('%Y%m%d'))
            print(f)
        else:
            print(name + ' ' + date.strftime('%Y%m%d') + ' exists')


# 用于读取招商估值表
def zs_gzb(file_path):
    data = pd.read_excel(file_path)
    name = data.iloc[1, 0].replace('招商证券_', '').replace('_专用表', '').replace('招商证券备机_', '')
    name = name.replace('私募证券投资基金', '').replace('私募投资基金', '').replace('私募基金', '')
    date = datetime.strptime(data.iloc[2, 0][-10:], '%Y-%m-%d').date()
    data.columns = data.iloc[3]
    data = data.iloc[7:].reset_index(drop=True)
    df = data.rename(
        columns={
            '成本-本币': '成本',
            '成本占比': '成本占净值',
            '行情': '市价',
            '市值-本币': '市值',
            '市值占比': '市值占净值',
            '估值增值-本币': '估值增值'
        }
    )
    df_col = df.columns.tolist()
    df['基金名称'] = name
    df['日期'] = date
    df['成本占净值'] = df['成本占净值'] * 100
    df['市值占净值'] = df['市值占净值'] * 100
    df = gzb_str_2_num(df)
    return df[['日期', '基金名称'] + df_col]


# 招商估值表录入数据库
def zs_gzb_to_db(table, fpath, db_path):
    data_exists = pd.read_sql_query('select `日期`, `基金名称` from ' + table, db_path)
    data_exists['label'] = data_exists['基金名称'] + data_exists['日期'].apply(lambda x: x.strftime('%Y%m%d'))
    data_exists = data_exists['label'].tolist()

    file_list = os.listdir(fpath)
    for f in file_list:
        file = fpath + '/' + f
        df = zs_gzb(file_path=file)[
            [
                '日期', '基金名称',
                '科目代码', '科目名称',
                '数量', '单位成本', '成本', '成本占净值',
                '市价', '市值', '市值占净值', '估值增值', '停牌信息',
            ]
        ]
        name = df['基金名称'][0]
        date = df['日期'][0]

        if name + date.strftime('%Y%m%d') not in data_exists:
            df.to_sql(table, db_path, index=False, if_exists='append')
            data_exists.append(name + date.strftime('%Y%m%d'))
            print(f)
        else:
            print(name + ' ' + date.strftime('%Y%m%d') + ' exists')


# 用于读取申万宏源估值表
def swhy_gzb(file_path):
    data = pd.read_excel(file_path)
    name = data.iloc[0, 0].replace('申万宏源证券有限公司___', '').replace('___专用表', '')
    name = name.replace('私募证券投资基金', '').replace('私募投资基金', '').replace('私募基金', '')
    date = datetime.strptime(data.iloc[1, 0][-10:], '%Y-%m-%d').date()
    data.columns = data.iloc[2]
    data = data.iloc[3:].reset_index(drop=True)
    df = data.rename(
        columns={
            '成本占净值(%)': '成本占净值',
            '市值占净值(%)': '市值占净值'
        }
    )
    df_col = df.columns.tolist()
    df['基金名称'] = name
    df['日期'] = date
    df = gzb_str_2_num(df)
    return df[['日期', '基金名称'] + df_col]


# 申万宏源估值表录入数据库
def swhy_gzb_to_db(table, fpath, db_path):
    data_exists = pd.read_sql_query('select `日期`, `基金名称` from ' + table, db_path)
    data_exists['label'] = data_exists['基金名称'] + data_exists['日期'].apply(lambda x: x.strftime('%Y%m%d'))
    data_exists = data_exists['label'].tolist()

    file_list = os.listdir(fpath)
    for f in file_list:
        file = fpath + '/' + f
        df = swhy_gzb(file_path=file).iloc[:, :13]
        name = df['基金名称'][0]
        date = df['日期'][0]

        if name + date.strftime('%Y%m%d') not in data_exists:
            df.to_sql(table, db_path, index=False, if_exists='append')
            data_exists.append(name + date.strftime('%Y%m%d'))
            print(f)
        else:
            print(name + ' ' + date.strftime('%Y%m%d') + ' exists')


